/************************************************************************************
TerraLib - a library for developing GIS applications.
Copyright  2001-2004 INPE and Tecgraf/PUC-Rio.

This code is part of the TerraLib library.
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

You should have received a copy of the GNU Lesser General Public
License along with this library.

The authors reassure the license terms regarding the warranties.
They specifically disclaim any warranties, including, but not limited to,
the implied warranties of merchantability and fitness for a particular purpose.
The library provided hereunder is on an "as is" basis, and the authors have no
obligation to provide maintenance, support, updates, enhancements, or modifications.
In no event shall INPE and Tecgraf / PUC-Rio be held liable to any party for direct,
indirect, special, incidental, or consequential damages arising out of the use
of this library and its documentation.
*************************************************************************************/

#include <stdio.h>
#include <direct.h>
#include <sys/stat.h>
#include <TeAdoOracle.h>
#include <sys/stat.h>

#include <TeUtils.h>

#define CHUNKSIZE	240

typedef map<int,TeNode> TeNodeMap;

inline void TESTHR( HRESULT hr )
{
	if( FAILED(hr) ) _com_issue_error( hr );
}

TeOracle::TeOracle()  
{
    HRESULT hr = CoInitialize(0);
	dbmsName_ = "OracleAdo";
    if(FAILED(hr))
    {
        cout << "Can't start COM!? " << endl;
    }
}

TeOracle::~TeOracle()
{
}

bool 
TeOracle::newDatabase(const string& database, const string& user, const string& password, 
					  const string& host, const int& port, bool terralibModel)
{
    if (!connect(host,user,password,database,port))
        return false;
    if (terralibModel)
    {
        //create conceptual model
        if(!this->createConceptualModel())  
			return false;
    }
     return true;
}


bool 
TeOracle::connect (const string& host, const string& user, const string& password, const string& database, int /* port */)
{
	
	string  connectionString = "Provider = OraOLEDB.Oracle.1;FetchSize=300;Password="+password+";Persist Security Info=True;User ID="+user+";Data Source="+database+"";

	try
	{
		connection_.CreateInstance(__uuidof(ADODB::Connection));
		HRESULT hr  = connection_->Open (connectionString.c_str(),"","",-1);
		TESTHR( hr );
	}
	catch(_com_error &e)
	{
		isConnected_ = false;
		errorMessage_ = e.Description();
		return false;
	}
	catch(...)
	{
		isConnected_ = false;
		errorMessage_ = "Oppps !";
		return false;
	}
	isConnected_ = true;
	database_ = database;
	host_ = host;
	user_ = user;
	password_ = password;
	return true;
}


bool
TeOracle::tableExist(const string& table)
{
	TeOraclePortal *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	string exist = " SELECT table_name FROM all_tables WHERE";
	exist += " TABLE_NAME = '" + TeConvertToUpperCase(table) + "'";
	exist += " AND OWNER = '" + TeConvertToUpperCase(user_) + "'";

	if(!ocip->query(exist))
	{
		delete ocip;
		return false;
	}

	if(ocip->fetchRow())
	{
		delete ocip;
		return true;
	}
	else
	{
		delete ocip;
		return false;
	}
}


bool 
TeOracle::listTables(vector<string>& tableList)
{
	tableList.clear();
	TeOraclePortal *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	string exist = " SELECT table_name FROM all_tables WHERE ";
	exist += " OWNER = '" + TeConvertToUpperCase(user_) + "'";

	if(!ocip->query(exist))
	{
		delete ocip;
		return false;
	}

	while(ocip->fetchRow())
		tableList.push_back (ocip->getData(0));
			
	delete ocip;
	return true;
}


bool 
TeOracle::columnExist(const string& table, const string& column, TeAttribute& attr)
{
	TeOraclePortal *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	string exist = " SELECT DATA_TYPE, DATA_LENGTH, DATA_SCALE FROM ALL_TAB_COLUMNS WHERE";
	exist += " TABLE_NAME = '" + TeConvertToUpperCase(table) + "'";
	exist += " AND COLUMN_NAME = '" + TeConvertToUpperCase(column) + "'";
	exist += " AND OWNER = '" + TeConvertToUpperCase(user_) + "'";

	if(!ocip->query(exist))
	{
		delete ocip;
		return false;
	}

	if(ocip->fetchRow())
	{	
		attr.rep_.name_ = column;
		
		string	dataType = string(ocip->getData(0)); 
		int		dataLength = atoi(ocip->getData(1)); 
		int		dataScale = atoi(ocip->getData(2));
		bool    number = false;
		
		/***** begin */
		/*
		if(dataType=="CBLOB") 
		{
			attr.rep_.type_ = TeSTRING;
			attr.rep_.numChar_ = dataLength;
		}
		else 
		*/
		/***** end */
		if(dataType=="VARCHAR2")
		{
			attr.rep_.type_ = TeSTRING;
			attr.rep_.numChar_ = dataLength;
		}
		else if (dataType=="BLOB")
		{
			attr.rep_.type_ = TeBLOB;
			attr.rep_.numChar_ = dataLength;
		}
		else if (dataType=="NUMBER")
		{
			number = true;
		}
		else if (dataType== "CHAR")
		{	
			attr.rep_.type_ = TeCHARACTER;
			attr.rep_.numChar_ = dataLength;
		}
		else if (dataType=="DATE")
		{
			attr.rep_.type_ = TeDATETIME;
		}
		else
		{
			attr.rep_.type_ = TeSTRING;
			attr.rep_.numChar_ = dataLength;
		}

		if(number)
		{
			if(dataScale > 0)
				attr.rep_.type_ = TeREAL;
			else
				attr.rep_.type_ = TeINT;
		}
		delete ocip;
		return true;
	}
	delete ocip;
	return false;
}


bool
TeOracle::createTable(const string& table, TeAttributeList &attr)
{
	short	cont=0;
	string	pkeys ="";
	bool	hasAutoNumber=false;
	string	fieldName="";

	if(tableExist(table))
	{
		errorMessage_= "Table already exist!";
		return false;
	}

	TeAttributeList::iterator it = attr.begin();
	string tablec;
	tablec = "CREATE TABLE " + table +" (";
	
	while ( it != attr.end())
	{
		if (cont)
			tablec += ", ";

		switch ((*it).rep_.type_)
		{
			case TeSTRING:
			   if ( (*it).rep_.numChar_ < 256 ) 
				  tablec += (*it).rep_.name_ + " VARCHAR2(" + Te2String((*it).rep_.numChar_) + ")";    
				else 
				  tablec += (*it).rep_.name_ + " VARCHAR(2000)";	
			break;
			
			case TeREAL:
				if((*it).rep_.decimals_>0)
					tablec += (*it).rep_.name_ +" NUMBER(*,"+ Te2String((*it).rep_.decimals_) +") ";
				else
					tablec += (*it).rep_.name_ +" NUMBER(*,38) ";
			break;
			
			case TeINT:
			case TeUNSIGNEDINT:
				tablec += (*it).rep_.name_ + " NUMBER(32) ";
			break;

			case TeDATETIME:
				tablec += (*it).rep_.name_ + " DATE ";
			break;

			case TeCHARACTER:
				tablec += (*it).rep_.name_ + " CHAR ";
			break;

			case TeBLOB:
				tablec += (*it).rep_.name_ + " BLOB ";
			break;

			default:
				tablec += (*it).rep_.name_ + " VARCHAR2(255) ";
			break;
		}
		//default values
		if(!((*it).rep_.defaultValue_.empty()))
			tablec += " DEFAULT '" + (*it).rep_.defaultValue_ + "' ";

		//not null values
		if(!((*it).rep_.null_))
			tablec += " NOT NULL ";

		// auto number
		if((*it).rep_.isAutoNumber_ && ((*it).rep_.type_==TeINT || (*it).rep_.type_==TeUNSIGNEDINT))  
		{
			hasAutoNumber=true;
			fieldName=(*it).rep_.name_;
		}

		// check if column is part of primary key
		if ((*it).rep_.isPrimaryKey_ && (*it).rep_.type_ != TeBLOB )
		{
			if (!pkeys.empty())
				pkeys += ", ";
			pkeys += (*it).rep_.name_;
			tablec += " NOT NULL";
		}

		++it;
		cont++;
	}

	if(!pkeys.empty())
		tablec += ", PRIMARY KEY (" + pkeys + ") ";

	tablec += ")";

	if(!execute(tablec))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table " + table;

		return false;
	}

	if(hasAutoNumber)
	{
		
		string dropSql = " DROP TRIGGER "+ getNameTrigger(table); 
		execute(dropSql);
		dropSql = " DROP SEQUENCE "+ getNameSequence(table); 
		execute(dropSql);

		if(!createSequence(table))
		{
			deleteTable(table);
			return false;
		}
		
		if(!createAutoIncrementTrigger(table,fieldName))
		{
			deleteTable(table);
			string sql= "DROP SEQUENCE "+ getNameSequence(table);
			execute(sql); 
			return false;
		}
	}

	return true;
}


bool
TeOracle::deleteTable (const string& table)
{
//	int f =	table.find ("te_collection", std::string::npos);
	int f =	table.find ("te_collection");

	if( table=="te_theme" ||
		table=="te_layer" ||
		table=="te_representation" ||
		table=="te_tables_relation" ||
		table=="te_layer_table" ||
		table=="te_color_scheme" ||
		table=="te_raster_metadata" ||
		table=="te_projection" ||
		table=="te_view" ||
		table=="te_chart_params" ||
		table=="te_legend" ||
		table=="te_visual" ||
		f == 0)
	{
		errorMessage_ = "No  possvel deletar tabelas do modelo!";
		return false;
	}

	string del = "DROP TABLE " + table;
	if(!execute(del))
		return false;

	string seq = "DROP SEQUENCE " + getNameSequence(table);
	execute(seq);
		
	string trig = "DROP TRIGGER " + getNameTrigger(table);
	execute(trig);
		
	return true;
}

bool 
TeOracle::addColumn (const string& table, TeAttributeRep &rep)
{
	if(!tableExist(table))
		return false;

	string tab;
	tab = " ALTER TABLE " + table + " ADD ( ";
	tab += rep.name_ + "  ";
	
	switch (rep.type_)
	{
		case TeSTRING:
			if (rep.numChar_<256)
				tab  += "VARCHAR2(" + Te2String(rep.numChar_) + ") ";
			else
				tab += "VARCHAR(2000) ";
			break;
		
		case TeREAL:
			if(rep.decimals_>0)
				tab += " NUMBER(*," + Te2String(rep.decimals_) +") ";
			else
				tab += " NUMBER(*,38) ";
		break;
			
		case TeINT:
			tab += "NUMBER(32)";
			break;

		case TeDATETIME:
			tab += "DATE";

		case TeCHARACTER:
			tab += "CHAR";
		
		case TeBLOB:
			tab += "BLOB";
		
		default:
			tab += "VARCHAR2(" + Te2String(rep.numChar_) + ") ";
			break;
	}
	if (rep.isAutoNumber_)
		tab += " AUTO_INCREMENT)";
	else
		tab += " NULL)";

	if(!execute(tab))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error inserting a column to table " + table + " !";
		return false;
	}

	alterTableInfoInMemory(table);
	return true;
}

bool 
TeOracle::alterTable (const string& table, TeAttributeRep &rep, const string& oldColName)
{
	if(!tableExist(table))
		return false;

	if(!oldColName.empty() && oldColName != rep.name_ ) //rename column
	{
		string add = " ALTER TABLE "+ table +" RENAME COLUMN "+ oldColName +" TO "+ rep.name_;
		if(!execute(add))
		{
			if(errorMessage_.empty())
				errorMessage_ = "Error alter table " + table + " !";
			return false;
		}
	}

	string tab = " ALTER TABLE " + table + " MODIFY ( ";
	tab += rep.name_ + "  ";
	switch (rep.type_)
	{
		case TeSTRING:
			if ( rep.numChar_ < 256 )
				tab  += "VARCHAR2(" + Te2String(rep.numChar_) + ") ";
			else
				tab += "VARCHAR(2000) "; 
			break;

		case TeREAL:
			if(rep.decimals_>0)
				tab += " NUMBER(*,"+ Te2String(rep.decimals_) +") ";
			else
				tab += " NUMBER(*,38) ";
		break;
		
		case TeINT:
			tab += " NUMBER(32) ";
			break;

		case TeDATETIME:
			tab += " DATE ";
			break;

		case TeCHARACTER:
			tab += " CHAR ";
			break;
		
		default:
			tab += " VARCHAR2(" + Te2String(rep.numChar_) + ") ";
			break;
	}

	tab += " )";

	if(!execute(tab))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error alter table " + table + " !";
		return false;
	}
	
	string tableId;
	TeDatabasePortal* portal = getPortal();
	string sql = "SELECT table_id FROM te_layer_table WHERE attr_table = '" + table + "'";
	if(portal->query(sql) && portal->fetchRow())
		tableId = portal->getData(0);

	delete portal;

	if(tableId.empty() == false)
	{
		if(oldColName.empty() == false) // column name changed
		{
			 // update relation
			sql = "UPDATE te_tables_relation SET related_attr = '" + rep.name_ + "'";
			sql += " WHERE related_table_id = " + tableId;
			sql += " AND related_attr = '" + oldColName + "'";
			if(execute(sql) == false)
				return false;

			sql = "UPDATE te_tables_relation SET external_attr = '" + rep.name_ + "'";
			sql += " WHERE external_table_name = '" + table + "'";
			sql += " AND external_attr = '" + oldColName + "'";
			if(execute(sql) == false)
				return false;

			 // update grouping
			sql = "UPDATE te_grouping SET grouping_attr = '" + table + "." + rep.name_ + "'";
			sql += " WHERE grouping_attr = '" + table + "." + oldColName + "'";
			if(execute(sql) == false)
				return false;
		}
		else // column type changed
		{
			// delete relation
			sql = "DELETE FROM te_tables_relation WHERE (related_table_id = " + tableId;
			sql += " AND related_attr = '" + rep.name_ + "')";
			sql += " OR (external_table_name = '" + table + "'";
			sql += " AND external_attr = '" + rep.name_ + "')";
			if(execute(sql) == false)
				return false;

			// delete grouping
			TeDatabasePortal* portal = getPortal();
			sql = "SELECT theme_id FROM te_grouping WHERE grouping_attr = '" + table + "." + oldColName + "'";
			if(portal->query(sql) && portal->fetchRow())
			{
				string themeId = portal->getData(0);

				sql = "DELETE FROM te_legend WHERE theme_id = " + themeId + " AND group_id >= 0";
				if(execute(sql) == false)
				{
					delete portal;
					return false;
				}
			}
			delete portal;

			sql = "DELETE FROM te_grouping";
			sql += " WHERE grouping_attr = '" + table + "." + oldColName + "'";
			if(execute(sql) == false)
				return false;
		}
	}
	alterTableInfoInMemory(table);
	return true;
}

bool 
TeOracle::alterTable(const string& oldTableName, const string& newTablename)
{
	string sql = " ALTER TABLE "+ oldTableName +" RENAME TO "+ newTablename;
	if(!this->execute(sql))
		return false;

	//update te_layer_table
	sql = " UPDATE te_layer_table ";
	sql += " SET attr_table = '"+ newTablename +"'";
	sql += " WHERE attr_table = '"+ oldTableName +"'";
	execute(sql);

	//update te_tables_relation
	sql = " UPDATE te_tables_relation ";
	sql += " SET external_table_name = '"+ newTablename +"'";
	sql += " WHERE external_table_name = '"+ oldTableName +"'";
	execute(sql);
	
	alterTableInfoInMemory(newTablename, oldTableName);
	return true;
}


bool
TeOracle::createRelation (const string& name, const string& table, const string& fieldName, const string& relatedTable, const string& relatedField, bool cascadeDeletion)
{
	
	string relation = "ALTER TABLE " + table + " ADD ";
	relation += " CONSTRAINT " + name;
	relation += " FOREIGN KEY ( " + fieldName + ") "; 
	relation += " REFERENCES " + relatedTable + "(" + relatedField + ")";
		
	if (cascadeDeletion)
		relation += " ON DELETE CASCADE ";

	if(!execute(relation))
		return false;

	return true;
}


TeDBRelationType 
TeOracle::existRelation(const string& tableName, const string& relName)
{

	TeOraclePortal  *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return TeNoRelation;
	
	string integ = " SELECT DELETE_RULE  ";
	integ += " FROM   all_constraints ";
	integ += " WHERE  CONSTRAINT_TYPE = 'R'"; 
	integ += " AND    OWNER = '" + TeConvertToUpperCase(user_) + "'";
	integ += " AND    CONSTRAINT_NAME = '" + TeConvertToUpperCase(relName) + "'";
	
	string temp = " AND    TABLE_NAME = '" + TeConvertToUpperCase(tableName) + "' ";   
	integ += temp;
	
	if(!ocip->query(integ))
	{
		delete ocip;
		return TeNoRelation;
	}

	string cascade;
	if(ocip->fetchRow())
	{
		cascade = string(ocip->getData(0));
		if(cascade== "CASCADE")
		{
			delete ocip;
			return TeRICascadeDeletion;
		}
		
		delete ocip;
		return TeRINoCascadeDeletion;
	}

	delete ocip;
	return TeNoRelation;	
}


TeDatabasePortal*  
TeOracle::getPortal ()
{
	TeOraclePortal* portal = new TeOraclePortal (this);
	return portal;
}

bool 
TeOracle::createDatabaseTable()
{
	string create = "CREATE TABLE te_database (";
	create += "db_version		VARCHAR(50) NOT NULL,";
	create += "db_creation		DATE,";
	create += "PRIMARY KEY (db_version))";
	return (execute (create));
}

bool 
TeOracle::createProjectionTable()
{

	string table;
	table = "CREATE TABLE te_projection (";
	table += "projection_id ";
	table += "NUMBER(32) NOT NULL";
	table += ", name ";
	table += "VARCHAR2(255) NOT NULL";
	table += ", long0 ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL ";
	table += ", lat0 ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL ";
	table += ", offx ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL ";
	table += ", offy ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL ";
	table += ", stlat1 ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL ";
	table += ", stlat2 ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL ";
	table += ", unit ";
	table += "VARCHAR2(100) ";
	table += ", scale ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL ";
	table += ", hemis ";
	table += "NUMBER(32) ";
	table += ", datum ";
	table += "VARCHAR2(100) ";
	table += ", radius ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL ";
	table += ", flattening ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL ";
	table += ", dx ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL ";
	table += ", dy ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL ";
	table += ", dz ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL ";
	
	table += ", PRIMARY KEY (projection_id))";

	if(!execute(table))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table te_projection!";
		return false;
	}
	
	//creates a sequence to table 
	if(!createSequence("te_projection"))
	{
		deleteTable("te_projection");
		return false;
	}

	return true;
}

bool
TeOracle::createAutoIncrementTrigger(const string &tableName, const string &fieldName)
{
	string nameTri = getNameTrigger(tableName);
	string nameSeq = getNameSequence(tableName);

	string tri;
	tri = "CREATE TRIGGER " + nameTri; 
	tri += " BEFORE INSERT ON "+tableName; 
	tri += " for each row";
	tri += " begin";
	tri += " select "+nameSeq+".nextval";
	tri += " into :new."+fieldName;
	tri += " from dual;";
	tri += " end;";

	if(!execute(tri))
	{	
		if(errorMessage_.empty())
			errorMessage_ = "Error creating trigger to table " + tableName + " !";
		return false;
	}
	return true;
}


bool
TeOracle::createSequence(const string &tableName)
{
	string nameSeq = getNameSequence(tableName);
	
	string seq = " CREATE SEQUENCE " + nameSeq;
	seq += " START WITH 1 INCREMENT BY 1 ORDER ";
	if (!execute(seq))
	{	
		if(errorMessage_.empty())
			errorMessage_ = "Error creating sequence to table " + tableName + " !";
		return false;
	}

	return true;
}

string
TeOracle::getNameSequence(const string &tableName)
{
	string name;
	if(tableName.size()>21)
		name = tableName.substr(0,20) + "_seq";
	else
		name = tableName + "_seq";

	return name;
}

string
TeOracle::getNameTrigger(const string &tableName)
{
	string name;
	if(tableName.size()>21)
		name = tableName.substr(0,20) + "_tri";
	else
		name = tableName + "_tri";

	return name;
}


bool 
TeOracle::createLayerTable()
{
	string table;
	table = "CREATE TABLE te_layer (";
	table += "layer_id ";
	table += "NUMBER(32) NOT NULL ";
	table += ", projection_id ";
	table += "NUMBER(32) NOT NULL ";
	table += "CONSTRAINT layer_proj UNIQUE ";
	table += ", name ";
	table += "VARCHAR2(255) NOT NULL ";
	table += ", lower_x ";
	table += "NUMBER(*,38)";
	table += ", lower_y ";
	table += "NUMBER(*,38)";
	table += ", upper_x ";
	table += "NUMBER(*,38)";
	table += ", upper_y ";
	table += "NUMBER(*,38)";
	table += ", initial_time ";
	table += "DATE ";
	table += ", final_time ";
	table += "DATE ";

	table += ", PRIMARY KEY (layer_id))";

	if(!execute(table))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table te_layer!";
		return false;
	}

	//creates a sequence to table 
	if(!createSequence("te_layer"))
	{
		deleteTable("te_layer");
		return false;
	}

	return true;
}

bool 
TeOracle::createLayerTableTable()
{
	string table;
	table = "CREATE TABLE te_layer_table (";
	table += "table_id ";
	table += " NUMBER(32) NOT NULL ";
	table += ", layer_id ";
	table += "NUMBER(32) ";
	table += ", attr_table ";
	table += "VARCHAR2(255) NOT NULL ";
	table += ", unique_id ";
	table += "VARCHAR2(255) ";
	table += ", attr_link ";
	table += "VARCHAR2(255) ";
	table += ", attr_initial_time ";
	table += "VARCHAR2(255) ";
	table += ", attr_final_time ";
	table += "VARCHAR2(255) ";
	table += ", attr_time_unit ";
	table += "NUMBER(32) ";
	table += ", attr_table_type ";
	table += "NUMBER(32) ";
	table += ", user_name ";
	table += "VARCHAR2(255) ";
	table += ", initial_time ";
	table += "DATE ";
	table += ", final_time ";
	table += "DATE ";

	table += ", PRIMARY KEY (table_id))";

	if(!execute(table))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table te_layer_table!";
		return false;
	}

	//creates a sequence to table 
	if(!createSequence("te_layer_table"))
	{
		deleteTable("te_layer_table");
		return false;
	}

	return true;
}

bool 
TeOracle::createTablesRelationTable()
{

	string table;
	table = "CREATE TABLE te_tables_relation (";
	table += "relation_id ";
	table += "NUMBER(32) NOT NULL ";
	table += ", related_table_id ";
	table += "NUMBER(32) NOT NULL ";
	table += ", related_attr ";
	table += "VARCHAR2(255) NOT NULL";
	table += ", external_table_name ";
	table += "VARCHAR2(255) NOT NULL";
	table += ", external_attr ";
	table += "VARCHAR2(255) NOT NULL ";

	table += ", PRIMARY KEY (relation_id))";

	if(!execute(table))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table te_tables_relation!";
		return false;
	}

	//creates a sequence to table 
	if(!createSequence("te_tables_relation"))
	{
		deleteTable("te_tables_relation");
		return false;
	}

	return true;

}

bool 
TeOracle::createRepresentationTable ()
{
	string table;
	table = "CREATE TABLE te_representation (";
	table += " repres_id ";
	table += " NUMBER(32) NOT NULL "; 
	table += ", layer_id ";
	table += "NUMBER(32) NOT NULL";
	table += ", geom_type ";
	table += "NUMBER(32) NOT NULL";
	table += ", geom_table ";
	table += "VARCHAR2(255) NOT NULL";
	table += ", description "; 
	table += "VARCHAR2(255) ";
	table += ", lower_x ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL";
	table += ", lower_y ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL";
	table += ", upper_x ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL";
	table += ", upper_y ";
	table += "NUMBER(*,38) DEFAULT 0.0000 NOT NULL";
	table += ", res_x ";
	table += "NUMBER(*,38) ";
	table += ", res_y ";
	table += "NUMBER(*,38) ";
	table += ", num_cols ";
	table += "NUMBER(32)";
	table += ", num_rows ";
	table += "NUMBER(32)";
	table += ", initial_time ";
	table += "DATE";
	table += ", final_time ";
	table += "DATE";
	
	table += ", PRIMARY KEY (repres_id))";
 
	if(!execute(table))
	{	
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table te_representation!";
		return false;
	}

	//creates a sequence to table 
	if(!createSequence("te_representation"))
	{
		deleteTable("te_representation");
		return false;
	}

	return true;
}

bool
TeOracle::createRasterMetadataTable (const string& tableName)
{
	
	if(tableExist(tableName))
	{
		errorMessage_= "Table already exist!";
		return false;
	}
	
	string table;
	table = "CREATE TABLE "+ tableName +" (";
	table += " geom_id ";
	table += "NUMBER(32) NOT NULL";
	table += ", band_id ";
	table += "NUMBER(32) NOT NULL";
	table += ", min_value ";
	table += "NUMBER(*,38) DEFAULT 0.0 ";
	table += ", max_value ";
	table += "NUMBER(*,38) DEFAULT 0.0 ";
	table += ", num_bits ";
	table += "NUMBER(32)";
	table += ", data_type ";
	table += "NUMBER(32)";
	table += ", photometric_type ";
	table += "NUMBER(32)";
	table += ", compression_type ";
	table += "NUMBER(32)";
	table += ", dummy ";
	table += "NUMBER(*,38) ";
	
	table += ", PRIMARY KEY (geom_id, band_id))";

	if(!execute(table))
	{	
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table "+ tableName +"!";
		return false;
	}
	
	return true;
}


bool
TeOracle::createLUTTable(const string& table)
{
	if(tableExist(table))
	{
		errorMessage_= "Table already exist!";
		return false;
	}
	
	string tab;
	tab = "CREATE TABLE " + table + " ( ";
	tab += "index_id ";
	tab += "NUMBER(32) NOT NULL ";
	tab += ", r_val ";
	tab += "NUMBER(32) NOT NULL ";
	tab += ", g_val ";
	tab += "NUMBER(32) NOT NULL ";
	tab += ", b_val ";
	tab += "NUMBER(32) NOT NULL ";
	
	tab += ", PRIMARY KEY (index_id))";

	if(!execute(tab))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table " + table ;
		return false;
	}
	return true;
}

bool 
TeOracle::createViewTable()
{
	string table;
	table = "CREATE TABLE te_view ( ";
	table += "view_id ";
	table += "NUMBER(32) NOT NULL";
	table += ", projection_id ";
	table += "NUMBER(32) NOT NULL";
	table += " CONSTRAINT view_proj UNIQUE ";
	table += ", name ";
	table += "VARCHAR2(255) NOT NULL";
	table += ", user_name ";
	table += "VARCHAR2(255) ";
	table += ", visibility ";
	table += "NUMBER(32) ";

	table += ", PRIMARY KEY(view_id))";
	
	if(!execute(table))
	{	
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table te_view!";
		return false;
	}
	
	//creates a sequence to table 
	if(!createSequence("te_view"))
	{
		deleteTable("te_view");
		return false;
	}

	//creates index
	if (!createIndex ("te_view", "view_index", "name, user_name"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table te_view!";
		return false;
	}
	
	return true;
}

bool 
TeOracle::createThemeTable ()
{
	string table;
	table = "CREATE TABLE te_theme (";
	table += "theme_id ";
	table += "NUMBER(32) NOT NULL";
	table += ", layer_id ";
	table += "NUMBER(32) NULL";
	table += ", view_id ";
	table += "NUMBER(32) NULL";
	table += ", name ";
	table += "VARCHAR2(255) NOT NULL";
	table += ", parent_id ";
	table += "NUMBER(32) ";
	table += ", priority ";
	table += "NUMBER(32) ";	
	table += ", node_type ";
	table += "NUMBER(32) NOT NULL";	
		
	table += ", min_scale ";
	table += "NUMBER(*,38)";
	table += ", max_scale ";
	table += "NUMBER(*,38)";

	table += ", generate_attribute_where ";
	table += "VARCHAR2(500)";
	table += ", generate_spatial_where ";
	table += "VARCHAR2(500)";
	table += ", generate_temporal_where ";
	table += "VARCHAR2(500)";
	
	table += ", collection_table ";
	table += "VARCHAR2(255)";
	
	table += ", visible_rep ";
	table += "NUMBER(32) NULL";
	table += ", enable_visibility ";
	table += "NUMBER(32) NULL";

	table += ", PRIMARY KEY (theme_id))";
	
	if(!execute(table))
	{	
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table te_theme!";
		return false;
	}

	//creates a sequence to the tabela
	if(!createSequence("te_theme"))
	{
		deleteTable("te_theme");
		return false;
	}

	return true;
}

bool 
TeOracle::createThemeTablesTable()
{
	string table = "CREATE TABLE te_theme_table  (";
	table += " theme_table_id ";
	table += " NUMBER(32) NOT NULL";
	table += ", theme_id ";
	table += " NUMBER(32) NOT NULL ";
	table += ", table_id ";
	table += " NUMBER(32) NOT NULL ";
	table += ", relation_id ";
	table += " NUMBER(32)";
	table += ", table_order ";
	table += " NUMBER(32) ";
	table += ", PRIMARY KEY (theme_table_id) ";
	table += ")";
	
	if(!execute(table))
	{	
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table te_theme_table!";
		return false;
	}
	
	//creates a sequence to table 
	if(!createSequence("te_theme_table"))
	{
		deleteTable("te_theme_table");
		return false;
	}
	
	//creates index
	if (!createIndex ("te_theme_table", "theme_table_index", "theme_id"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table te_theme_table!";
		return false;
	}

	return true;
}

bool
TeOracle::createGroupingTable()
{
	string table;
	table = "CREATE TABLE te_grouping (";
	table += "theme_id ";
	table += "NUMBER(32) NOT NULL";
	table += ", grouping_number ";
	table += "NUMBER(32)";
	table += ", grouping_attr ";
	table += "VARCHAR2(255)";
	table += ", grouping_attr_type ";
	table += "NUMBER(32)";
	table += ", grouping_mode ";
	table += "NUMBER(32)";
	table += ", grouping_norm_attr ";
	table += "VARCHAR2(255)";
	table += ", grouping_std_dev ";
	table += "NUMBER(*,38) DEFAULT 0.0";
	table += ", grouping_precision ";
	table += "NUMBER(32)";
	table += ", grouping_function ";
	table += "VARCHAR2(20)";
	table += ", grouping_chronon ";
	table += "NUMBER(32)";
	table += ", PRIMARY KEY (theme_id))";
	
	if(!execute(table))
	{	
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table te_grouping!";
		return false;
	}
	return true;
}


bool 
TeOracle::createLegendTable ()
{
	string table;
	table = "CREATE TABLE te_legend (";
	table += "  legend_id ";
	table += "NUMBER(32) NOT NULL";
	table += ", theme_id ";
	table += "NUMBER(32) NOT NULL";
	table += ", group_id ";
	table += "NUMBER(32)";
	table += ", num_objs ";
	table += "NUMBER(32)";
	table += ", lower_value ";
	table += "VARCHAR2(255)";
	table += ", upper_value ";
	table += "VARCHAR2(255)";
	table += ", label ";
	table += "VARCHAR2(255)";

	table += ", PRIMARY KEY (legend_id))";

	if(!execute(table))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table te_legend!";
		return false;
	}
	
	//creates a sequence to the table 
	if(!createSequence("te_legend"))
	{
		deleteTable("te_legend");
		return false;
	}

	//creates index
	if (!createIndex ("te_legend", "legend_index", "theme_id"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table te_legend!";
		return false;
	}

	return true;
}

bool 
TeOracle::createVisualTable ()
{
	string table;
	table = "CREATE TABLE te_visual (";
	table += "  legend_id ";
	table += "NUMBER(32) NOT NULL";
	table += ", geom_type ";
	table += "NUMBER(32) NOT NULL";

	table += ", lib_name ";
	table += "VARCHAR2(255) ";
	table += ", symb_id ";
	table += "NUMBER(32) ";
	table += ", red ";
	table += "NUMBER(32) ";
	table += ", green ";
	table += "NUMBER(32) ";
	table += ", blue ";
	table += "NUMBER(32) ";
	table += ", transparency ";
	table += "NUMBER(32) ";
	table += ", width ";
	table += "NUMBER(32) ";
	
	table += ", contour_lib_name ";
	table += "VARCHAR2(255) ";
	table += ", contour_symb_id ";
	table += "NUMBER(32) ";
	table += ", contour_red ";
	table += "NUMBER(32) ";
	table += ", contour_green ";
	table += "NUMBER(32) ";
	table += ", contour_blue ";
	table += "NUMBER(32) ";
	table += ", contour_transp ";
	table += "NUMBER(32) ";
	table += ", contour_width ";
	table += "NUMBER(32) ";

	table += ", size_value ";
	table += "NUMBER(32) ";
	table += ", pt_angle ";
	table += "NUMBER(32) ";
	
	table += ", family ";
	table += "VARCHAR2(100)";
	table += ", bold ";
	table += "NUMBER(32) ";
	table += ", italic ";
	table += "NUMBER(32) ";
	table += ", alignment_vert ";
	table += " NUMBER(*,38) ";
	table += ", alignment_horiz ";
	table += " NUMBER(*,38) ";
	table += ", tab_size ";
	table += "NUMBER(32) ";
	table += ", line_space ";
	table += "NUMBER(32) ";
	table += ", fixed_size ";
	table += "NUMBER(32) ";

#ifdef USE_STYLE_LIB
	table += ", point_style ";
	table += "BLOB ";
	table += ", line_style ";
	table += "BLOB ";
	table += ", polygon_style ";
	table += "BLOB ";
#endif

	table += ", PRIMARY KEY (legend_id, geom_type))";

	if(!execute(table))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table te_visual!";
		return false;
	}
	
	return true;
}

bool 
TeOracle::createVisualRasterTable()
{
	string create = "CREATE TABLE te_visual_raster (";
	create += " theme_id		NUMBER(32) NOT NULL,";
	create += " band_in			NUMBER(32) NOT NULL,";
	create += " band_out		NUMBER(32), ";
	create += " transf_type		NUMBER(32), ";
	create += " param1			NUMBER(*,38),";
	create += " param2			NUMBER(*,38),";
	create += " lut_table		VARCHAR2(255), ";
	create += " PRIMARY KEY (theme_id, band_in))";
	
	if(!execute(create))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table te_visual_raster!";
		return false;
	}
	
	return true;
}

bool 
TeOracle::createCollectionTable(const string& tableName)
{

	if(tableExist(tableName))
	{
		errorMessage_= "Table already exist!";
		return false;
	}
	
	string table = "CREATE TABLE "+ tableName +" (";
	table += " c_object_id ";
	table += "VARCHAR2(255) NOT NULL";
	table += ", c_legend_id  ";
	table += "NUMBER(32) ";
	table += ", label_x ";
	table += " NUMBER(*,38) ";
	table += ", label_y ";
	table += " NUMBER(*,38) ";
	table += ", c_legend_own ";
	table += "NUMBER(32) ";
	table += ", c_object_status ";
	table += "NUMBER(32) ";
	
	table += ", PRIMARY KEY (c_object_id))";

	if(!execute(table))
	{	
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table " + tableName + "!";
		return false;
	}
		
	return true;
}


bool 
TeOracle::createCellGeometry (const string& table)
{
	
	if(tableExist(table))
	{
		errorMessage_= "Table already exist!";
		return false;
	}
	
	string tab;
	tab = "CREATE TABLE " + table + "(";
	tab += "geom_id ";
	tab += "NUMBER(32) NOT NULL";
	tab += ", object_id ";
	tab += "VARCHAR2(255) NULL";
	tab += ", col_number ";
	tab += "NUMBER(32) NOT NULL";
	tab += ", row_number ";
	tab += "NUMBER(32) NOT NULL";
	tab += ", lower_x ";
	tab += "NUMBER(*,38)";
	tab += ", lower_y ";
	tab += "NUMBER(*,38)";
	tab += ", upper_x ";
	tab += "NUMBER(*,38)";
	tab += ", upper_y ";
	tab += "NUMBER(*,38)";
	tab += ", PRIMARY KEY (geom_id))";

	if (!execute(tab))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table " + table + " !" ;
		return false;
	}

	//creates indexes
	if(!createIndex(table, table + "obj_idx", "object_id"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}

	if(!createIndex(table, table + "box_idx", "lower_x, lower_y, upper_x, upper_y"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}


	if(!createIndex(table, table + "rc_idx", "row_number, col_number"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}

	//creates a sequence to table 
	if(!createSequence(table))
	{
		deleteTable(table);
		return false;
	}

	if(!createAutoIncrementTrigger(table,"geom_id"))
	{
		deleteTable(table);
		string sql= "DROP SEQUENCE "+ getNameSequence(table);
		execute(sql); 
		return false;
	}

	return true;
}

bool 
TeOracle::createTextGeometry (const string& table)
{
	
	if(tableExist(table))
	{
		errorMessage_= "Table already exist!";
		return false;
	}
	
	string tab;
	tab = "CREATE TABLE " + table +"(";
	tab += "geom_id ";
	tab += "NUMBER(32) NOT NULL";
	tab += ", object_id ";
	tab += "VARCHAR2(255) ";
	tab += ", x ";
	tab += "NUMBER(*,38) NULL";
	tab += ", y ";
	tab += "NUMBER(*,38) NULL";
	tab += ", text_value ";
	tab += "VARCHAR2(255) NULL";
	tab += ", angle ";
	tab += "NUMBER(*,38) NULL";
	tab += ", height ";
	tab += "NUMBER(*,38) NULL";
	tab += ", alignment_vert ";
	tab += "NUMBER(*,38) NULL";
	tab += ", alignment_horiz ";
	tab += "NUMBER(*,38) NULL";
	
	tab += ", PRIMARY KEY (geom_id))";

	if(!execute(tab))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table  " + table + " !";
		return false;
	}

	//creates indexes
	if(!createIndex(table, table + "obj_idx", "object_id"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}

	if(!createIndex(table, table + "xy_idx", "x,y"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}
	
	//creates a sequence to table 
	if(!createSequence(table))
	{
		deleteTable(table);
		return false;
	}

	if(!createAutoIncrementTrigger(table, "geom_id"))
	{
		deleteTable(table);
		string sql= "DROP SEQUENCE "+ getNameSequence(table);
		execute(sql); 
		return false;
	}

	return true;
}


bool 
TeOracle::createArcGeometry (const string& table)
{
	
	if(tableExist(table))
	{
		errorMessage_= "Table already exist!";
		return false;
	}

	string tab;
	tab = "CREATE TABLE " + table +"(";
	tab += "geom_id ";
	tab += "NUMBER(32) NOT NULL";
	tab += ", object_id ";
	tab += "VARCHAR2(255) NULL";
	tab += ", from_node ";
	tab += "NUMBER(32) NOT NULL";
	tab += ", to_node ";
	tab += "NUMBER(32) NOT NULL";
	
	tab += ", PRIMARY KEY (geom_id))";

	if(!execute(tab))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table " + table ;
		return false;
	}

	//creates indexes
	if(!createIndex(table, table + "obj_idx", "object_id"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}

	if(!createIndex(table, table + "fn_idx", "from_node"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}

	if(!createIndex(table, table + "tn_idx", "to_node"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}
		
	//creates a sequence to table 
	if(!createSequence(table))
	{
		deleteTable(table);
		return false;
	}

	if(!createAutoIncrementTrigger(table,"geom_id"))
	{
		deleteTable(table);
		string sql= "DROP SEQUENCE "+ getNameSequence(table);
		execute(sql); 
		return false;
	}
	
	return true;
}

bool 
TeOracle::createNodeGeometry (const string& table)
{
	
	if(tableExist(table))
	{
		errorMessage_= "Table already exist!";
		return false;
	}
		
	string tab;
	tab = "CREATE TABLE " + table + " (";
	tab += "geom_id ";
	tab += "NUMBER(32) NOT NULL ";
	tab += ", object_id ";
	tab += "VARCHAR2(255) NULL";
	tab += ", x ";
	tab += "NUMBER(*,38) NOT NULL";
	tab += ", y ";
	tab += "NUMBER(*,38) NOT NULL";
	tab += ", PRIMARY KEY (geom_id))";

	if(!execute(tab))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table  " + table + " !";
		return false;
	}
	
	//creates indexes
	if(!createIndex(table, table + "obj_idx", "object_id"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}

	if(!createIndex(table, table + "xy_idx", "x,y"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}
		
	//creates a sequence to table 
	if(!createSequence(table))
	{
		deleteTable(table);
		return false;
	}

	if(!createAutoIncrementTrigger(table,"geom_id"))
	{
		deleteTable(table);
		string sql= "DROP SEQUENCE "+ getNameSequence(table);
		execute(sql); 
		return false;
	}
	
	return true;
}

bool 
TeOracle::createRasterGeometry (const string& table)
{
	
	if(tableExist(table))
	{
		errorMessage_= "Table already exist!";
		return false;
	}

	string create = "CREATE TABLE " + table + " (";
	create += "geom_id		NUMBER(32) NOT NULL,";
	create += "object_id	VARCHAR2(255) NOT NULL,";
	create += "raster_table VARCHAR2(255) NOT NULL,";
	create += "lut_table	VARCHAR2(255) ,";
	create += "res_x		NUMBER(*,38) DEFAULT 0.0,";
	create += "res_y		NUMBER(*,38) DEFAULT 0.0,";
	create += "num_bands	NUMBER(32),";
	create += "num_cols		NUMBER(32),";
	create += "num_rows		NUMBER(32),";
	create += "block_height NUMBER(32),";
	create += "block_width	NUMBER(32),";
	create += "lower_x		NUMBER(*,38) DEFAULT 0.0,";
	create += "lower_y		NUMBER(*,38) DEFAULT 0.0,";
	create += "upper_x		NUMBER(*,38) DEFAULT 0.0,";
	create += "upper_y		NUMBER(*,38) DEFAULT 0.0,";
	create += "tiling_type  NUMBER(32),";
	create += "PRIMARY KEY (geom_id))";
	
	if(!execute (create))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table " + table + " !" ;
		return false;
	}
	
	if(!createIndex(table, table + "obj_idx", "object_id"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}
		
	if(!createSequence(table))
	{
		deleteTable(table);
		return false;
	}

	if(!createAutoIncrementTrigger(table,"geom_id"))
	{
		deleteTable(table);
		string sql= "DROP SEQUENCE "+ getNameSequence(table);
		execute(sql); 
		return false;
	}
	
	return true;
}

bool 
TeOracle::createRasterTable (const string& tableName)
{
	string tab;
	tab = "CREATE TABLE "+ tableName + " ( ";
	tab += "block_id ";
	tab += " VARCHAR2(50) NOT NULL";
	tab += ", band_id ";
	tab += " NUMBER(32) NOT NULL";
	tab += ", resolution_factor ";
	tab += " NUMBER(32) ";
	tab += ", subband ";
	tab += " NUMBER(32) ";
	tab += ", lower_x ";
	tab += " NUMBER(*,38) DEFAULT 0.0 ";
	tab += ", lower_y ";
	tab += " NUMBER(*,38) DEFAULT 0.0 ";
	tab += ", upper_x ";
	tab += " NUMBER(*,38) DEFAULT 0.0 ";
	tab += ", upper_y ";
	tab += " NUMBER(*,38) DEFAULT 0.0 ";
	tab += ", block_size ";
	tab += " NUMBER(32) ";
	tab += ", spatial_data ";
	tab += " BLOB ";
	
	tab += ", PRIMARY KEY (block_id))";

	if(!execute(tab))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table " + tableName ;
		return false;
	}
	
	if(!createIndex(tableName, tableName + "box_idx", "lower_x, lower_y, upper_x, upper_y, resolution_factor, subband"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + tableName + " !" ;
		return false;
	}	

	if(!createIndex(tableName, tableName + "band_idx", "band_id"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + tableName + " !" ;
		return false;
	}

	return true;
}

bool
TeOracle::deleteLayer(int layerId)
{
	//Delete attributes tables
	if(!deleteLayerTable(layerId))
		return false;

	TeOraclePortal* portal = (TeOraclePortal*) this->getPortal();
	if (!portal)
		return false;

	string geomTable;
	string sql = "SELECT projection_id FROM te_layer WHERE layer_id = ";
	sql += Te2String(layerId);

	if (!portal->query(sql))
	{	
		delete portal;
		return false;
	}

	if (!portal->fetchRow())
	{
		delete portal;
		return false;
	}
	string projId = portal->getData("projection_id");
	portal->freeResult();

	// Get all representations that are associated to this layer
	sql = "SELECT * FROM te_representation WHERE layer_id = "+ Te2String(layerId);
	if (!portal->query (sql))
	{
		delete portal;
		return false;
	}

	while (portal->fetchRow())
	{	// Delete the geometry tables
		geomTable = portal->getData("geom_table");
	
		// Delete lut table
		TeGeomRep rep = TeGeomRep(atoi(portal->getData("geom_type")));
		if (rep == TeRASTER)
		{
			TeOraclePortal* portal2 = (TeOraclePortal*)this->getPortal();
			sql = "SELECT lut_table, raster_table FROM " + geomTable;
			string tabName;
			if (!portal2->query (sql))
			{
				delete portal2;
				continue;
			}

			while (portal2->fetchRow())
			{
				// remove lut table
				tabName = portal2->getData(0);
				if (!tabName.empty() && this->tableExist(tabName))
				{
					sql = "DROP TABLE " + tabName;
					this->execute(sql);

					sql= "DROP SEQUENCE "+ getNameSequence(tabName);
					this->execute(sql); 
				}
				// remove raster table
				tabName = portal2->getData(1);
				if (!tabName.empty() && this->tableExist(tabName))
				{
					sql = "DROP TABLE " + tabName;
					this->execute(sql);

					sql= "DROP SEQUENCE "+ getNameSequence(tabName);
					this->execute(sql); 
				}
			}
			delete portal2;
			// remove raster metadata table
			tabName = geomTable + "_metadata";
			if (!tabName.empty() && this->tableExist(tabName))
			{
				sql = "DROP TABLE " + tabName;
				this->execute(sql);

				sql= "DROP SEQUENCE "+ getNameSequence(tabName);
				this->execute(sql); 
			}
		}
		if (this->tableExist(geomTable))
		{
			sql = "DROP TABLE " + geomTable;
			if (!this->execute(sql) )
			{
				delete portal;
				return false;
			}
			
			sql= "DROP SEQUENCE "+ getNameSequence(geomTable);
			this->execute(sql);
		}
	}

	portal->freeResult();
	if (existRelation("te_representation","fk_rep_layer_id") != TeRICascadeDeletion)
	{
		// Delete entries into representations table
		sql = "DELETE FROM te_representation WHERE layer_id = " +Te2String(layerId);
		if (!this->execute(sql) )
		{
			delete portal;
			return false;
		}
	}

	// delete layer themes
	sql = "SELECT theme_id FROM te_theme WHERE layer_id=" + Te2String(layerId);
	if (!portal->query (sql))
	{
		delete portal;
		return false;
	}
	
	int themeId;
	while (portal->fetchRow())
	{	
		themeId = atoi(portal->getData("theme_id"));
		this->deleteTheme(themeId);
	}
	
	sql = "DELETE FROM te_layer WHERE layer_id=" + Te2String(layerId);
	if (!this->execute(sql))
	{
		delete portal;
		return false;
	}

	// delete layer projection
	sql = "DELETE FROM te_projection WHERE projection_id = "+ projId;
	if (!this->execute(sql))
	{	
		delete portal;
		return false;
	}

	// remove all the items themes associated to the layer to be removed
	TeThemeMap::iterator it;
	for (it = themeMap_.begin(); it != themeMap_.end(); ++it)
	{
		TeTheme *theme = it->second;
		if (theme->layer()->id() == layerId)
		{
			themeMap_.erase(theme->id());
			delete theme;
		}
	}

	// Delete layer and its entry in the layer map
	TeLayer* layer = layerMap_[layerId];
	layerMap_.erase(layerId);
	delete layer;

	delete portal;
	return true;
}


bool 
TeOracle::createPolygonGeometry (const string& table)
{
	
	if(tableExist(table))
	{
		errorMessage_= "Table already exist!";
		return false;
	}
	
	string tab;
	tab = "CREATE TABLE " + table + " ( ";
	tab += "geom_id ";
	tab += "NUMBER(32) NOT NULL ";
	tab += ", object_id ";
	tab += "VARCHAR2(255) NULL";
	tab += ", num_coords ";
	tab += "NUMBER(32) NOT NULL ";
	tab += ", num_holes ";
	tab += "NUMBER(32) NOT NULL ";
	tab += ", parent_id ";
	tab += "NUMBER(32) NOT NULL ";
	tab += ", lower_x ";
	tab += "NUMBER(*,38) NOT NULL ";
	tab += ", lower_y ";
	tab += "NUMBER(*,38) NOT NULL ";
	tab += ", upper_x ";
	tab += "NUMBER(*,38) NOT NULL ";
	tab += ", upper_y ";
	tab += "NUMBER(*,38) NOT NULL ";
	tab += ", ext_max ";
	tab += "NUMBER(*,38) NOT NULL ";
	tab += ", spatial_data ";
	tab += "BLOB ";	
	tab += ", PRIMARY KEY (geom_id))";

	if(!execute(tab))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table " + table ;
		return false;
	}

	//creates indexes
	if(!createIndex(table, table + "obj_idx", "object_id"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}

	if(!createIndex(table, table + "parent_idx", "parent_id"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}

	if(!createIndex(table, table + "extm_idx", "ext_max"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}

	if(!createIndex(table, table + "box_idx", "lower_x, lower_y, upper_x, upper_y"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}

	//creates a sequence to table 
	if(!createSequence(table))
	{
		deleteTable(table);
		return false;
	}

	//creates trigger
	if(!createAutoIncrementTrigger(table, "geom_id"))
	{
		deleteTable(table);
		string sql= "DROP SEQUENCE "+ getNameSequence(table);
		execute(sql); 
		return false;
	}

	return true;
}

bool 
TeOracle::createLineGeometry (const string& table)
{
	
	if(tableExist(table))
	{
		errorMessage_= "Table already exist!";
		return false;
	}

	string tab;
	tab = "CREATE TABLE " + table + " ( ";
	tab += "geom_id ";
	tab += "NUMBER(32) NOT NULL ";
	tab += ", object_id ";
	tab += "VARCHAR2(255) NULL";
	tab += ", num_coords ";	
	tab += "NUMBER(32) NOT NULL ";
	tab += ", lower_x ";
	tab += "NUMBER(*,38) NOT NULL ";
	tab += ", lower_y ";
	tab += "NUMBER(*,38) NOT NULL ";
	tab += ", upper_x ";
	tab += "NUMBER(*,38) NOT NULL ";
	tab += ", upper_y ";
	tab += "NUMBER(*,38) NOT NULL ";
	tab += ", ext_max ";
	tab += "NUMBER(*,38) NOT NULL ";
	tab += ", spatial_data ";
	tab += "BLOB ";	
	tab += ", PRIMARY KEY (geom_id))";


	if(!execute(tab))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table " + table + " !" ;
		return false;
	}

	//creates indexes
	if(!createIndex(table, table + "obj_idx", "object_id"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}

	if(!createIndex(table, table + "box_idx", "lower_x, lower_y, upper_x, upper_y"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}
			
	//creates a sequence to table 
	if(!createSequence(table))
	{
		deleteTable(table);
		return false;
	}

	//creates trigger
	if(!createAutoIncrementTrigger(table, "geom_id"))
	{
		deleteTable(table);
		string sql= "DROP SEQUENCE "+ getNameSequence(table);
		execute(sql); 
		return false;
	}

	return true;
}

bool 
TeOracle::createPointGeometry (const string& table)
{
	
	if(tableExist(table))
	{
		errorMessage_= "Table already exist!";
		return false;
	}

	string tab;
	tab = "CREATE TABLE " + table + " ( ";
	tab += "geom_id ";
	tab += "NUMBER(32) NOT NULL";
	tab += ", object_id ";
	tab += "VARCHAR2(255) NULL";
	tab += ", x ";
	tab += "NUMBER(*,38) ";
	tab += ", y ";	
	tab += "NUMBER(*,38) ";
	tab += ", PRIMARY KEY (geom_id))";

	if(!execute(tab))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating table " + table + " !" ;
		return false;
	}	
		
	//creates indexes
	if(!createIndex(table, table + "obj_idx", "object_id"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}

	if(!createIndex(table, table + "xy_idx", "x,y"))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error creating index to table " + table + " !" ;
		return false;
	}
		
	//creates a sequence to table 
	if(!createSequence(table))
	{
		deleteTable(table);
		return false;
	}

	if(!createAutoIncrementTrigger(table,"geom_id"))
	{
		deleteTable(table);
		string sql= "DROP SEQUENCE "+ getNameSequence(table);
		execute(sql); 
		return false;
	}

	return true;
}

bool
TeOracle::insertProjection(TeProjection* proj)
{
	string insert = "INSERT INTO te_projection (projection_id, name, long0, lat0,";
	insert += " offx, offy, stlat1, stlat2, unit, scale, hemis, datum, ";
	insert += " radius, flattening, dx , dy, dz ) VALUES ( ";
	insert += "te_projection_seq.NEXTVAL";
	insert += ", '" + escapeSequence(proj->name()) + "'";
	insert += ", " + Te2String(proj->params().lon0*TeCRD);
	insert += ", " + Te2String(proj->params().lat0*TeCRD);
	insert += ", " + Te2String(proj->params().offx);
	insert += ", " + Te2String(proj->params().offy);
	insert += ", " + Te2String(proj->params().stlat1*TeCRD);
	insert += ", " + Te2String(proj->params().stlat2*TeCRD);
	insert += ", '" + escapeSequence(proj->params().units) + "'";
	insert += ", " + Te2String(proj->params().scale);
	insert += ", " + Te2String(proj->params().hemisphere);
	insert += ", '" + escapeSequence(proj->params().datum.name()) + "'";
	insert += ", " + Te2String(proj->params().datum.radius());
	insert += ", " + Te2String(proj->params().datum.flattening());
	insert += ", " + Te2String(proj->params().datum.xShift());
	insert += ", " + Te2String(proj->params().datum.yShift());
	insert += ", " + Te2String(proj->params().datum.zShift());
	insert += ")";

	if(!execute(insert))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error inserting in the table te_projection!";   
		return false;
	}

	TeOraclePortal *ocip = (TeOraclePortal*) getPortal();
	if(!ocip)
		return false;

	string seq = "SELECT te_projection_seq.CURRVAL FROM DUAL";
	if(!ocip->query(seq))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error in the sequence te_projection_seq!";  
		delete ocip;
		return false;
	}

	if(!ocip->fetchRow())
	{
		errorMessage_ = "Sequence value not found!";
		delete ocip;
		return false;
	}

	proj->id(atoi((const char*)ocip->getData(0))); 
	delete ocip;

	return true;
}

bool 
TeOracle::insertTableInfo(int layerId, TeTable &table, const string& user)
{
	string seq = getNameSequence("te_layer_table");
	string ins = "INSERT INTO te_layer_table ( "; 
	ins += " table_id, layer_id, attr_table, unique_id, attr_link, ";
	ins += " attr_initial_time, attr_final_time, attr_time_unit, ";
	ins += " attr_table_type, user_name) VALUES ( ";
	ins += seq + ".NEXTVAL ";
	
	if(layerId>0)
		ins += ", " + Te2String(layerId);
	else
		ins += ", NULL ";
	
	ins += ", '" + escapeSequence(table.name()) + "'";
	ins += ", '" + escapeSequence(table.uniqueName()) + "'";
	ins += ", '" + escapeSequence(table.linkName()) + "'";
	ins += ", '" + escapeSequence(table.attInitialTime()) + "'";
	ins += ", '" + escapeSequence(table.attFinalTime()) + "'";
	ins += ",  " + Te2String(table.attTimeUnit());
	ins += ",  " + Te2String(table.tableType());
	ins += ", '" +  escapeSequence(user) + "'";
	ins += ")";

	if(!execute(ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error inserting tables information!";   
		return false;
	}

	TeOraclePortal *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	ins = "SELECT "+ seq +".CURRVAL FROM DUAL";
	if (!ocip->query(ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error in the sequence te_layertable_seq!";  
		delete ocip;
		return false;
	}

	if(!ocip->fetchRow())
	{
		errorMessage_ = "Sequence value not found!";
		delete ocip;
		return false;
	}

	int index = atoi((const char*)ocip->getData(0));
	table.setId(index);
	delete ocip;

	return true;
}

bool 
TeOracle::insertRelationInfo(const int tableId, const string& tField,
						  const string& rName, const string& rField, int& relId)
{
	TeOraclePortal *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	relId = -1;

	relId = -1;
	string sel = "SELECT relation_id FROM te_tables_relation WHERE";
	sel += " related_table_id = " + Te2String(tableId);
	sel += " AND related_attr = '" + tField + "'";
	sel += " AND external_table_name = '" + rName + "'";
	sel += " AND external_attr = '" + rField + "'";
	if (!ocip->query(sel))
	{
		delete ocip;
		return false;
	}

	if (ocip->fetchRow())
	{
		relId = atoi(ocip->getData(0));
		delete ocip;
		return true;
	}

	string seq, ins;
	seq = getNameSequence("te_tables_relation");	
	ins = " INSERT INTO te_tables_relation ( "; 
	ins += " relation_id, related_table_id, ";
	ins += " related_attr, external_table_name, external_attr) VALUES ( ";
	ins += seq +".NEXTVAL "; 
	ins += ",  " + Te2String(tableId);
	ins += ", '" + escapeSequence(tField) + "'";
	ins += ", '" +  escapeSequence(rName) + "'";
	ins += ", '" +  escapeSequence(rField) + "'";
	ins += ")";

	if(!execute(ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error inserting tables information!";   
		delete ocip;
		return false;
	}

	ocip->freeResult();
	ins = "SELECT "+ seq +".CURRVAL FROM DUAL";
	if (!ocip->query(ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error in the sequence te_table_relation_seq!";  
		delete ocip;
		return false;
	}

	if(!ocip->fetchRow())
	{
		errorMessage_ = "Sequence value not found!";
		delete ocip;
		return false;
	}

	relId = atoi((const char*)ocip->getData(0));
	delete ocip;
	return true;
}


bool 
TeOracle::insertView (TeView *view)
{
		// save its projection
	TeProjection* proj = view->projection();
	if ( !proj || !insertProjection(proj))
	{
		errorMessage_ = "No  possvel inserir vista sem projeo";
		return false;
	}

	string ins = "INSERT INTO te_view (view_id, projection_id, name, user_name, visibility)";
	ins += " VALUES (";
	ins += "te_view_seq.NEXTVAL";
	ins += ", " + Te2String(proj->id());
	ins += ", '" + escapeSequence(view->name ()) + "'";
	ins += ", '" + escapeSequence(view->user ()) + "'";
	ins += ", " + Te2String((int)view->isVisible());
	ins += " )";
	
	if(!execute (ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error inserting in the table te_view!";   
		return false;
	}

	TeOraclePortal *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	ins = "SELECT te_view_seq.CURRVAL FROM DUAL";
	if (!ocip->query(ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error in the sequence te_view_seq!";  
		delete ocip;
		return false;
	}

	if(!ocip->fetchRow())
	{
		errorMessage_ = "Sequence value not found!";
		delete ocip;
		return false;
	}

	int index = atoi((const char*)ocip->getData(0));
	view->id(index);
	delete ocip;

	int size = view->size();

	for (int th=0; th<size; th++)
	{
		TeViewNode* node = view->get(th);
		if (node->type() == TeTHEME)
		{
			TeTheme *theme = (TeTheme*) node;
			insertTheme (theme);
		}
		else
		{
			TeViewTree* tree = (TeViewTree*)node;
			insertViewTree (tree);
		}
	}

	// Insert view in the view map
	viewMap_[view->id()] = view;
	return true;
}

bool
TeOracle::insertViewTree (TeViewTree *tree)
{
	
	string ins = "INSERT INTO te_theme (theme_id, view_id, name, ";
	ins += " parent_id, node_type, priority) VALUES (";
	ins += " te_theme_seq.NEXTVAL";
	ins += ", " + Te2String(tree->view());
	ins += ", '" + escapeSequence(tree->name()) + "'";
	ins += ", " + Te2String(tree->parentId());
	ins += ", " + Te2String(tree->type());
	ins += ", " + Te2String(tree->priority());
	ins += ")";
	
	if(!execute(ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error inserting in the table te_theme!";
		return false;
	}

	TeOraclePortal	*ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	string seq = "SELECT te_theme_seq.CURRVAL FROM DUAL";
	if(!ocip->query(seq))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error in the sequence te_theme_seq!";  
		delete ocip;
		return false;
	}

	if(!ocip->fetchRow())
	{
		errorMessage_ = "Sequence value not found!";
		delete ocip;
		return false;
	}

	int index = atoi((const char*)ocip->getData(0)); 
	tree->id(index);
	delete ocip;
	
	return true;
}


bool 
TeOracle::insertTheme (TeTheme *theme)
{

	double maxScale = theme->maxScale ();
	if(maxScale==TeMAXFLOAT)
		maxScale = 0.;

	string ins = "INSERT INTO te_theme (theme_id, layer_id, view_id, name, ";
	ins += " parent_id, priority, node_type, min_scale, max_scale, ";
	ins += " generate_attribute_where, generate_spatial_where, generate_temporal_where, ";
	ins += " collection_table, visible_rep, enable_visibility) VALUES (";
	ins += "te_theme_seq.NEXTVAL";
	ins += ", " + Te2String(theme->layerId());
	ins += ", " + Te2String(theme->view());
	ins += ", '" + escapeSequence(theme->name()) + "'";
	ins += ", " + Te2String(theme->parentId ());
	ins += ", " + Te2String(theme->priority());
	ins += ", " + Te2String(theme->type ());
	ins += ", " + Te2String (theme->minScale());
	ins += ", " + Te2String (maxScale);
	ins += ", '" + escapeSequence(theme->attributeRest()) + "'";
	ins += ", '" + escapeSequence(theme->spatialRest()) + "'";
	ins += ", '" + escapeSequence(theme->temporalRest()) + "'";
	ins += ", '" + escapeSequence(theme->collectionTable()) + "'";
	ins += ", " + Te2String(theme->visibleRep());
	ins += ", " + Te2String(theme->visibility());
	ins += ")";

	if(!execute(ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error inserting in the table te_theme!";   
		return false;
	}

	TeOraclePortal  *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	string seq = "SELECT te_theme_seq.CURRVAL FROM DUAL";
	if(!ocip->query(seq))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error in the sequence te_theme_seq!";  
		delete ocip;
		return false;
	}

	if(!ocip->fetchRow())
	{
		errorMessage_ = "Sequence value not found!";
		delete ocip;
		return false;
	}

	int index = atoi((const char*)ocip->getData(0)); 
	theme->id(index);
	delete ocip;
	string colName = theme->collectionTable(); 

	//insert collection name
	if(colName.empty())
	{
		colName = "te_collection_" + Te2String(index);
		theme->collectionTable(colName);	
		
		ins = "UPDATE te_theme SET ";
		ins += " collection_table = '" + escapeSequence(colName) + "'";
		ins += " WHERE theme_id = " + Te2String (index);
		
		if(!execute(ins))
		{
			if(errorMessage_.empty())
				errorMessage_ = "Error inserting in the table te_theme!";   
			return false;
		}
	}
	
	bool status;

	// insert grouping
	int numSlices = 0;
	if(theme->grouping().groupMode_ != TeNoGrouping)
	{
		if(!insertGrouping (theme->id(), theme->grouping()))
			return false;
		numSlices = theme->grouping().groupNumSlices_;
	}
	
	// insert legend
	theme->outOfCollectionLegend().group(-1); 
	theme->outOfCollectionLegend().theme(theme->id()); 
	status = insertLegend (&(theme->outOfCollectionLegend())); 
	if (!status)
		return status;

	theme->withoutDataConnectionLegend().group(-2); 
	theme->withoutDataConnectionLegend().theme(theme->id()); 
	status = insertLegend (&(theme->withoutDataConnectionLegend())); 
	if (!status)
		return status;

	theme->defaultLegend().group(-3); 
	theme->defaultLegend().theme(theme->id()); 
	status = insertLegend (&(theme->defaultLegend())); 
	if (!status)
		return status;

	theme->pointingLegend().group(-4); 
	theme->pointingLegend().theme(theme->id()); 
	status = insertLegend (&(theme->pointingLegend())); 
	if (!status)
		return status;

	theme->queryLegend().group(-5); 
	theme->queryLegend().theme(theme->id()); 
	status = insertLegend (&(theme->queryLegend())); 
	if (!status)
		return status;

	theme->queryAndPointingLegend().group(-6); 
	theme->queryAndPointingLegend().theme(theme->id()); 
	status = insertLegend (&(theme->queryAndPointingLegend())); 
	if (!status)
		return status;

	for (int i = 0; i < numSlices; i++)
	{
		theme->legend()[i].group(i);
		theme->legend()[i].theme(theme->id());
		status = insertLegend (&(theme->legend()[i]));
		if (!status)
			return status;
	}
	if (!status)
		return status;

	themeMap_[theme->id()] = theme;
	return updateThemeTable(theme);
}


bool 
TeOracle::insertThemeTable (int themeId, int tableId, int relationId, int tableOrder)
{
	string ins = "INSERT INTO te_theme_table ";
	ins += " (theme_table_id, theme_id, table_id, relation_id, table_order)";
	ins += " VALUES ( ";
	ins += getNameSequence("te_theme_table") +".NEXTVAL ";
	ins += ", "+ Te2String(themeId);
	ins += ", "+ Te2String(tableId);
	
	if(relationId>0)
		ins += ", "+ Te2String(relationId);
	else
		ins += ", NULL ";

	ins += ", "+ Te2String(tableOrder);
	ins += ")";

	if(!execute(ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error inserting in the table te_theme_table!";   
		return false;
	}

	return true;
}

bool 
TeOracle::insertThemeGroup(TeViewTree* tree)
{
	string ins = "INSERT INTO te_theme (theme_id, view_id, name, ";
	ins += " parent_id, priority, node_type ) VALUES (";
	ins += "te_theme_seq.NEXTVAL";
	ins += ", " + Te2String(tree->view());
	ins += ", '" + escapeSequence(tree->name()) + "'";
	ins += ", " + Te2String(tree->parentId ());
	ins += ", " + Te2String(tree->priority());
	ins += ", " + Te2String(1);
	ins += ")";

	TeOraclePortal  *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;
	
	try
	{
		if(!execute(ins))
		{
			if(errorMessage_.empty())
				errorMessage_ = "Error inserting in the table te_theme!";   
			return false;
		}

		string seq = "SELECT te_theme_seq.CURRVAL FROM DUAL";
		if(!ocip->query(seq))
		{
			if(errorMessage_.empty())
				errorMessage_ = "Error in the sequence te_theme_seq!";  
			delete ocip;
			return false;
		}

		if(!ocip->fetchRow())
		{
			errorMessage_ = "Sequence value not found!";
			delete ocip;
			return false;
		}

		int index = atoi((const char*)ocip->getData(0)); 
		tree->id(index);
		
	}
	catch(...)
	{
		errorMessage_ = "Error inserting in the table te_theme!";
		if(ocip)
			delete ocip;
		return false;
	}

	delete ocip;
	return true;
}

bool 
TeOracle::generateLabelPositions (TeTheme *theme)
{
	string	geomTable, upd;
	string	collTable = theme->collectionTable();
	
	if((collTable.empty()) || (!tableExist(collTable)))
		return false;

	if( theme->layer()->hasGeometry(TeCELLS)    || 
		theme->layer()->hasGeometry(TePOLYGONS) ||
		theme->layer()->hasGeometry(TeLINES) )
	{
		geomTable = theme->layer()->tableName(TeCELLS);
		
		if(geomTable.empty())
		{
			geomTable = theme->layer()->tableName(TePOLYGONS);
			if(geomTable.empty())
				geomTable = theme->layer()->tableName(TeLINES);
		}

		upd= "UPDATE " + collTable + " SET ";
		upd += " label_x = (SELECT MAX(lower_x + (upper_x - lower_x)/2) ";
		upd += "FROM " + geomTable + " WHERE object_id = c_object_id), ";
		
		upd += " label_y = (SELECT MAX(lower_y + (upper_y - lower_y)/2) ";
		upd += "FROM " + geomTable + " WHERE object_id = c_object_id) ";
		upd += " WHERE label_x IS NULL OR label_y IS NULL";
	}
	else if(theme->layer()->hasGeometry(TePOINTS))
	{
		geomTable = theme->layer()->tableName(TePOINTS);
		
		upd= " UPDATE " + collTable + " SET ";
		upd += " label_x = (SELECT MAX(x) ";
		upd += " FROM " + geomTable + " p WHERE object_id = c_object_id), ";
		
		upd += " label_y = (SELECT MAX(y) ";
		upd += " FROM " + geomTable + " p WHERE object_id = c_object_id) ";
		upd += " WHERE label_x IS NULL OR label_y IS NULL";
	}

	if(!execute(upd))
		return false;

	return true;
}


bool 
TeOracle::insertLayer (TeLayer* layer)
{
	
	int index;
	TeProjection* proj = layer->projection();
	if (!proj || !insertProjection(proj))
	{
		errorMessage_ = "No  possvel inserir layer sem projeo";
		return false;
	}
	
	string ins = "INSERT INTO te_layer (layer_id, projection_id, name ";
	ins += ", lower_x, lower_y, upper_x, upper_y) ";
	ins += " VALUES ( ";
	ins += "te_layer_seq.NEXTVAL";
	ins += ", " + Te2String(proj->id());
	ins += ", '" + escapeSequence(layer->name()) + "'";
	ins += ", " + Te2String(layer->box().x1_);
	ins += ", " + Te2String(layer->box().y1_);
	ins += ", " + Te2String(layer->box().x2_);
	ins += ", " + Te2String(layer->box().y2_);
	ins += ")";

	if(!execute(ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error inserting in the table te_layer!";   
		return false;
	}

	TeOraclePortal *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	ins = "SELECT te_layer_seq.CURRVAL FROM DUAL";
	if (!ocip->query(ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error in the sequence te_layer_seq!";  
		delete ocip;
		return false;
	}

	if(!ocip->fetchRow())
	{
		errorMessage_ = "Sequence value not found!";
		delete ocip;
		return false;
	}

	index = atoi((const char*)ocip->getData(0));
	layer->id(index);
	delete ocip;
	layerMap_[layer->id()] = layer;
	return true;
}

bool 
TeOracle::insertRepresentation (int layerId, TeRepresentation& rep)
{
	if (layerId <= 0)
		return false;
	
	string ins;
	TeOraclePortal *ocip = (TeOraclePortal*) getPortal();
	if(!ocip)
		return false;

	try
	{
		ins = " INSERT INTO te_representation (repres_id, layer_id, geom_type, geom_table, ";
		ins += " description, lower_x, lower_y, upper_x, upper_y, res_x, res_y, num_cols, ";
		ins += " num_rows) VALUES (";
		ins += " te_representation_seq.NEXTVAL ";
		ins += ", " + Te2String(layerId);
		ins += ", " + Te2String(static_cast<int>(rep.geomRep_));
		ins += ", '" + escapeSequence(rep.tableName_) + "'";
		ins += ", '" + escapeSequence(rep.description_) + "'";
		ins += ", " + Te2String(rep.box_.x1_);
		ins += ", " + Te2String(rep.box_.y1_);
		ins += ", " + Te2String(rep.box_.x2_);
		ins += ", " + Te2String(rep.box_.y2_);
		ins += ", " + Te2String(rep.resX_);
		ins += ", " + Te2String(rep.resY_);
		ins += ", " + Te2String(rep.nCols_);
		ins += ", " + Te2String(rep.nLins_);
		ins += ")";

		if(!execute(ins))
		{
			if(errorMessage_.empty())
				errorMessage_ = "Error inserting in the table te_representation!";   
			return false;
		}
		
		string seq = "SELECT te_representation_seq.CURRVAL FROM DUAL";
		if(!ocip->query(seq))
		{
			if(errorMessage_.empty())
				errorMessage_ = "Error in the sequence te_representation_seq!";  
			delete ocip;
			return false;
		}

		if(!ocip->fetchRow())
		{
			errorMessage_ = "Sequence value not found!";
			delete ocip;
			return false;
		}

		int id = atoi((const char*)ocip->getData(0)); 
		rep.id_ = id;
	}

	catch(...)
	{
		errorMessage_ = "Error inserting in the table te_representation!"; 
		delete ocip;
		return false;
	}

	delete ocip;
	return true;
}	


bool 
TeOracle::insertLegend (TeLegendEntry* leg)
{
	TeOraclePortal *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	try
	{
		string ins = "INSERT INTO te_legend (legend_id, theme_id, group_id, ";
		ins += " num_objs, lower_value, upper_value, label) VALUES ( ";
		ins += "te_legend_seq.NEXTVAL";
		ins += ", " + Te2String(leg->theme());
		ins += ", " + Te2String(leg->group());
		ins += ", " + Te2String(leg->count());
		ins += ", '" + escapeSequence(leg->from()) + "'";
		ins += ", '" + escapeSequence(leg->to()) + "'";
		ins += ", '" + escapeSequence(leg->label()) + "'";
		ins += ")";
			
		if (!execute(ins))
		{
			if(errorMessage_.empty())
				errorMessage_ = "Error inserting in the table te_legend!"; 
			return false;
		}

		string seq = "SELECT te_legend_seq.CURRVAL FROM DUAL";
		if(!ocip->query(seq))
		{
			if(errorMessage_.empty())
				errorMessage_ = "Error in the sequence te_theme_seq!";  
			delete ocip;
			return false;
		}

		if(!ocip->fetchRow())
		{
			errorMessage_ = "Sequence value not found!";;
			delete ocip;
			return false;
		}

		int index = atoi((const char*)ocip->getData(0)); 
		leg->id(index);
	}
	catch(...)
	{
		errorMessage_ = "Error inserting in the table te_legend!"; 
		delete ocip;
		return false;
	}
		
	delete ocip;
	legendMap_[leg->id()] = leg;
	return insertVisual(leg);
}

bool 
TeOracle::insertPolygonSet (const string& table, TePolygonSet &ps)
{
	
	for (unsigned int i = 0; i < ps.size(); i++ )
	{
		TePolygon poly = ps [i];
		if (!insertPolygon (table,poly))
			return false;
	}
	return true;
}


bool 
TeOracle::insertPolygon (const string& table, TePolygon &poly)
{
	
	ADODB::_RecordsetPtr recset_;
	recset_.CreateInstance(__uuidof(ADODB::Recordset));
	
	recset_->CursorLocation=ADODB::adUseServer;
	
	recset_->Open(_bstr_t(table.c_str()),
		_variant_t((IDispatch*)connection_,true),
		ADODB::adOpenKeyset, 
		ADODB::adLockOptimistic,
		ADODB::adCmdTable);
	
	TeOraclePortal  *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	double	extmax;
	unsigned int k, ni, size;
//	double	*points	= NULL;

	try
	{
		int parentId = 0;
		for ( k = 0; k < poly.size(); k++ )
		{
			TeLinearRing ring ( poly[k] );
			TeBox b = ring.box();
			size = ring.size();
			ni = 0;
			extmax = MAX(b.width(),b.height());
			if (k==0)
				ni = poly.size()-1;
			else
				ni = 0;
			recset_->AddNew();
			recset_->Fields->GetItem("object_id")->Value = (_bstr_t) (poly.objectId().c_str());
			recset_->Fields->GetItem("num_coords")->Value = (_variant_t) ((long)size);
			recset_->Fields->GetItem("num_holes")->Value = (_variant_t) ((long)ni);
			recset_->Fields->GetItem("parent_id")->Value = (_variant_t) ((long)parentId);
			recset_->Fields->GetItem("lower_x")->Value = (_variant_t) (b.lowerLeft().x());
			recset_->Fields->GetItem("lower_y")->Value = (_variant_t) (b.lowerLeft().y());
			recset_->Fields->GetItem("upper_x")->Value = (_variant_t) (b.upperRight().x());
			recset_->Fields->GetItem("upper_y")->Value = (_variant_t) (b.upperRight().y());
			recset_->Fields->GetItem("ext_max")->Value = (_variant_t) (extmax);

			// Create and save the BLOB
			VARIANT varPoints;

			if(!LinearRingToVariant(varPoints, ring))
			{
				recset_->Close();
				delete ocip;
				return false;
			}

			//recset_->Fields->GetItem("spatial_data")->Value = (_variant_t) (varPoints);
			recset_->Fields->Item["spatial_data"]->AppendChunk (varPoints);
			recset_->Update();
			
			string seq = "SELECT " + getNameSequence(table)+ ".CURRVAL FROM DUAL";
			if(!ocip->query(seq))
			{
				if(errorMessage_.empty())
					errorMessage_ = "Error in the sequence " + table+ "!";  
				delete ocip;
				recset_->Close();
				return false;
			}

			if(!ocip->fetchRow())
			{
				errorMessage_ = "Sequence value not found!";
				recset_->Close();
				delete ocip;
				return false;
			}

			int index = atoi((const char*)ocip->getData(0)); 
			ring.geomId (index);
			ocip->freeResult();
			
			//ring.geomId (recset_->GetCollect("geom_id").intVal);			
			SafeArrayDestroy (varPoints.parray);
			if (k==0)
			{
				parentId = ring.geomId();
				string sql = "UPDATE "+table+" SET parent_id  ="+Te2String(parentId)+" WHERE geom_id = "+Te2String(ring.geomId());
				if(!execute(sql))
				{
					if(errorMessage_.empty())
						errorMessage_ = "Error inserting in table " + table;
					delete ocip;
					recset_->Close();
					return false;
				}
				
			}
			
		} //for
	}
	catch(_com_error &e)
	{
		errorMessage_ = e.Description();
		recset_->Close();
		delete ocip;
		return false;
	}
	catch(...)
	{
		errorMessage_ = "Oppps !";
		recset_->Close();
		delete ocip;
		return false;
	}

	delete ocip;
	recset_->Close();
	return true;
}

bool 
TeOracle::insertLineSet(const string& table, TeLineSet &ls)
{		
	ADODB::_RecordsetPtr recset_;
	recset_.CreateInstance(__uuidof(ADODB::Recordset));
		
	recset_->CursorLocation=ADODB::adUseServer;
	recset_->CacheSize = 300;
	recset_->Open(_bstr_t(table.c_str()),
		_variant_t((IDispatch*)connection_,true),
		ADODB::adOpenStatic,	
		ADODB::adLockBatchOptimistic,
		ADODB::adCmdTable);
	
	double	extmax;
	int		size;
//	double	*points	= NULL;

	for ( unsigned int i = 0; i < ls.size(); i++ )
	{
		TeLine2D line = ls [i];
		try
		{
			size = line.size();
			TeBox b = line.box();
			extmax = MAX(b.width(),b.height());
			recset_->AddNew();
			recset_->Fields->GetItem("object_id")->Value = (_bstr_t) (line.objectId().c_str());
			recset_->Fields->GetItem("num_coords")->Value = (_variant_t) ((long)size);
			recset_->Fields->GetItem("lower_x")->Value = (_variant_t) (b.lowerLeft().x());
			recset_->Fields->GetItem("lower_y")->Value = (_variant_t) (b.lowerLeft().y());
			recset_->Fields->GetItem("upper_x")->Value = (_variant_t) (b.upperRight().x());
			recset_->Fields->GetItem("upper_y")->Value = (_variant_t) (b.upperRight().y());
			recset_->Fields->GetItem("ext_max")->Value = (_variant_t) (extmax);

			// Create and save the BLOB
			VARIANT varPoints;

			if (!LineToVariant(varPoints, line))
			{
				recset_->Close();
				return false;
			}

			recset_->Fields->Item["spatial_data"]->AppendChunk (varPoints);
			//recset_->Update();
			
			SafeArrayDestroy (varPoints.parray);
		}
		catch(_com_error &e)
		{
			errorMessage_ = e.Description();
			recset_->Close();
			return false;
		}
		catch(...)
		{
			errorMessage_ = "Oppps !";
			recset_->Close();
			return false;
		}
	}
	
	recset_->UpdateBatch(ADODB::adAffectAll);
	recset_->Close();
	return true;
}


bool 
TeOracle::insertLine (const string& table, TeLine2D &line)
{
	ADODB::_RecordsetPtr recset_;
	recset_.CreateInstance(__uuidof(ADODB::Recordset));
	
	recset_->CursorLocation=ADODB::adUseServer;
	recset_->Open(_bstr_t(table.c_str()),
		_variant_t((IDispatch*)connection_,true),
		ADODB::adOpenKeyset, 
		ADODB::adLockOptimistic,
		ADODB::adCmdTable);

	double	extmax;
	int		size;
//	double	*points	= NULL;

	try
	{
		size = line.size();
		TeBox b = line.box();
		extmax = MAX(b.width(),b.height());
		recset_->AddNew();
		recset_->Fields->GetItem("object_id")->Value = (_bstr_t) (line.objectId().c_str());
		recset_->Fields->GetItem("num_coords")->Value = (_variant_t) ((long)size);
		recset_->Fields->GetItem("lower_x")->Value = (_variant_t) (b.lowerLeft().x());
		recset_->Fields->GetItem("lower_y")->Value = (_variant_t) (b.lowerLeft().y());
		recset_->Fields->GetItem("upper_x")->Value = (_variant_t) (b.upperRight().x());
		recset_->Fields->GetItem("upper_y")->Value = (_variant_t) (b.upperRight().y());
		recset_->Fields->GetItem("ext_max")->Value = (_variant_t) (extmax);
		
		// Create and save the BLOB
		VARIANT varPoints;
		if (!LineToVariant(varPoints, line))
		{
			recset_->Close();
			return false;
		}
		
		//recset_->Fields->GetItem("spatial_data")->Value = (_variant_t) (varPoints);		
		recset_->Fields->Item["spatial_data"]->AppendChunk (varPoints);
		recset_->Update();

		SafeArrayDestroy (varPoints.parray);
	}
	catch(_com_error &e)
	{
		recset_->Close();
		errorMessage_ = e.Description();
		return false;
	}
	catch(...)
	{
		recset_->Close();
		errorMessage_ = "Oppps !";
		return false;
	}
	recset_->Close();
	return true;
}

bool 
TeOracle::insertPointSet (const string& table, TePointSet &ps)
{
	for (unsigned int i = 0; i < ps.size(); i++ )
	{
		TePoint point = ps [i];
		if (!insertPoint (table,point))
			return false;
	}
	return true;
}

bool 
TeOracle::insertPoint (const string& table, TePoint &p)
{
	string ins = "INSERT INTO " + table + " ( ";
	ins += " geom_id, object_id, x, y) ";
	ins += " VALUES ( ";
	ins += getNameSequence(table) +".NEXTVAL";
	ins += ", '" + escapeSequence(p.objectId()) + "'";
	ins += ",  " + Te2String(p.location().x_);
	ins += ",  " + Te2String(p.location().y_);
	ins += " ) ";

	if(!execute(ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error inserting in the table " + table + "!"; 
		return false;
	}

	TeOraclePortal *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	string seq = "SELECT "+ getNameSequence(table) + ".CURRVAL FROM DUAL";
	if(!ocip->query(seq))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error in the sequence " + table + "_seq!";  
		delete ocip;
		return false;
	}

	if(!ocip->fetchRow())
	{
		errorMessage_ = "Sequence value not found!";
		delete ocip;
		return false;
	}

	int index = atoi((const char*)ocip->getData(0)); 
	p.geomId(index); 
	delete ocip;
	return true;
}


bool 
TeOracle::insertCellSet (const string& table, TeCellSet &cs)
{
	for (unsigned int i = 0; i < cs.size(); i++ )
	{
		TeCell cell = cs [i];
		if (!insertCell (table,cell))
			return false;
	}
	return true;
}


bool 
TeOracle::insertCell (const string& table, TeCell &cell)
{
	TeBox b = cell.box();
	
	string ins = "INSERT INTO " + table + " ( ";
	ins += " geom_id, object_id, col_number, row_number, lower_x, ";
	ins += " lower_y, upper_x, upper_y ) ";
	ins += " VALUES ( ";
	ins += getNameSequence(table) +".NEXTVAL";
	ins += ", '" + escapeSequence(cell.objectId ()) + "'";
	ins += ",  " + Te2String(cell.column ());
	ins += ",  " + Te2String(cell.line ());
	ins += ",  " + Te2String(b.lowerLeft().x_);
	ins += ",  " + Te2String(b.lowerLeft().y_);
	ins += ",  " + Te2String(b.upperRight().x_);
	ins += ",  " + Te2String(b.upperRight().y_);
	ins += " )";
		
	if(!execute(ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error inserting in the table " + table + "!"; 
		return false;
	}

	TeOraclePortal *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	string seq = "SELECT "+ getNameSequence(table) +".CURRVAL FROM DUAL";
	if(!ocip->query(seq))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error in the sequence " + table + "_seq!";  
		delete ocip;
		return false;
	}

	if(!ocip->fetchRow())
	{
		errorMessage_ = "Sequence value not found!";
		delete ocip;
		return false;
	}

	int index = atoi((const char*)ocip->getData(0)); 
	cell.geomId(index); 
	delete ocip;
	
	return true;
}

bool 
TeOracle::insertTextSet (const string& table, TeTextSet &ts)
{
	for (unsigned int i = 0; i < ts.size(); i++ )
	{
		TeText text = ts [i];
		if (!insertText (table,text))
			return false;
	}
	return true;
}

bool 
TeOracle::insertText(const string& table, TeText &text)
{
	string ins = "INSERT INTO " + table + " (geom_id, ";
	ins += " object_id, x, y, text_value, angle, height, alignment_vert, ";
	ins += " alignment_horiz) VALUES ( ";
	ins += getNameSequence(table) +".NEXTVAL";
	ins += ", '" + escapeSequence(text.objectId()) + "'";
	ins += ",  " + Te2String(text.location().x_);
	ins += ",  " + Te2String(text.location().y_);
	ins += ", '" + escapeSequence(text.textValue()) + "'";
	ins += ",  " + Te2String(text.angle());
	ins += ",  " + Te2String(text.height());
	ins += ",  " + Te2String(text.alignmentVert());
	ins += ",  " + Te2String(text.alignmentHoriz());
	ins += " )";
	
	if(!execute(ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error inserting in the table " + table + "!"; 
		return false;
	}

	TeOraclePortal *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	string seq = "SELECT "+ getNameSequence(table) +".CURRVAL FROM DUAL";
	if(!ocip->query(seq))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error in the sequence " + table + "_seq!";  
		delete ocip;
		return false;
	}

	if(!ocip->fetchRow())
	{
		errorMessage_ = "Sequence value not found!";
		delete ocip;
		return false;
	}

	int index = atoi((const char*)ocip->getData(0)); 
	text.geomId(index); 
	delete ocip;
	return true;
}

bool 
TeOracle::insertArcSet (const string& table, TeArcSet &as)
{
	for (unsigned int i = 0; i < as.size(); i++ )
	{
		TeArc arc = as [i];
		if (!insertArc (table,arc))
			return false;
	}
	return true;
}

bool 
TeOracle::insertArc (const string& table, TeArc &arc)
{
	string ins = "INSERT INTO " + table + " (geom_id, ";
	ins += " object_id, from_node, to_node ) ";
	ins += " VALUES ( ";
	ins += getNameSequence(table) +".NEXTVAL";
	ins += ", '" + escapeSequence(arc.objectId()) + "'";
	ins += ",  " + Te2String(arc.fromNode().geomId());
	ins += ",  " + Te2String(arc.toNode().geomId());
	ins += " )";
	
	if(!execute(ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error inserting in the table " + table + "!"; 
		return false;
	}

	TeOraclePortal *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	string seq = "SELECT "+ getNameSequence(table) +".CURRVAL FROM DUAL";
	if(!ocip->query(seq))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error in the sequence " + table + "_seq!";  
		delete ocip;
		return false;
	}

	if(!ocip->fetchRow())
	{
		errorMessage_ = "Sequence value not found!";
		delete ocip;
		return false;
	}

	int index = atoi((const char*)ocip->getData(0)); 
	arc.geomId(index); 
	delete ocip;
	return true;
}

bool 
TeOracle::insertNodeSet (const string& table, TeNodeSet &ns)
{
	for (unsigned int i = 0; i < ns.size(); i++ )
	{
	     TeNode no = ns [i];
	     if (!insertNode (table,no))
		return false;
	}
	return true;
}

bool 
TeOracle::insertNode (const string& table, TeNode &node)
{
	string ins = "INSERT INTO " + table + " ( ";
	ins += " geom_id, object_id, x, y) ";
	ins += " VALUES ( ";
	ins += getNameSequence(table) +".NEXTVAL";
	ins += ", '" + escapeSequence(node.objectId()) + "'";
	ins += ",  " + Te2String(node.location().x_);
	ins += ",  " + Te2String(node.location().y_);
	ins += " ) ";

	if(!execute(ins))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error inserting in the table " + table + "!"; 
		return false;
	}

	TeOraclePortal *ocip = (TeOraclePortal*)getPortal();
	if(!ocip)
		return false;

	string seq = "SELECT "+ getNameSequence(table) + ".CURRVAL FROM DUAL";
	if(!ocip->query(seq))
	{
		if(errorMessage_.empty())
			errorMessage_ = "Error in the sequence " + table + "_seq!";  
		delete ocip;
		return false;
	}

	if(!ocip->fetchRow())
	{
		errorMessage_ = "Sequence value not found!";
		delete ocip;
		return false;
	}

	int index = atoi((const char*)ocip->getData(0)); 
	node.geomId(index); 
	delete ocip;
	return true;
}


bool 
TeOracle::insertTable (TeTable &table)
{
	string tableName = table.name();
	int size = table.size();
	TeAttributeList att = table.attributeList();
	TeAttributeList::iterator it = att.begin();
	TeAttributeList::iterator itEnd = att.end();
	TeTableRow row;
	int i;
	unsigned int j;
	for ( i = 0; i < size; i++ )	
	{
		row = table[i];
		it = att.begin();
		string attrs;
		string values;
		j = 1;
		int jj = 0;
		while ( it != itEnd )
		{
			if (row[jj].empty() || (*it).rep_.isAutoNumber_)
			{
				++it;
				j++;
				jj++;
				continue;
			}			
			if (!values.empty())
			{
				attrs += ", ";
				values += ", ";
			}			
			attrs += (*it).rep_.name_;			
			switch ((*it).rep_.type_)
			{
			case TeSTRING:
				values += "'"+ escapeSequence( row[jj] ) +"'";
				break;
			case TeREAL:
				{
				std::string strValue = row[jj];
				replace(strValue.begin(), strValue.end(), ',', '.');
				values += strValue;
				}
				break;
			case TeINT:
				values += row[jj];
				break;
			case TeDATETIME:
				{
					TeTime t  (row[jj], (*it).dateChronon_, (*it).dateTimeFormat_,  
						      (*it).dateSeparator_, (*it).timeSeparator_, (*it).indicatorPM_);
					values += " TO_DATE('" + t.getDateTime() + "', 'DD/MM/YYYY HH24:MI:SS')";
				}
				break;
			case TeCHARACTER:
				values += "'" + escapeSequence(row[jj]) + "'";
				break;
			case TeBLOB:
				values += " EMPTY_BLOB() ";
				break;
			default:
				values += "'"+ escapeSequence(row[jj]) +"'";
				break;
			}
			++it;
			j++;
			jj++;
		}

		if (values.empty()) 
			continue;

		string q = "INSERT INTO "+tableName + " ("+ attrs+") " + " VALUES ("+values+") ";
		if (!execute(q)) 
			continue;
	}
	return true;
}

string 
TeOracle::getSQLAutoNumber(const string& table)
{
	string aut = getNameSequence(table) +".NEXTVAL";
	return aut;
}


bool 
TeOracle::getAttributeList(const string& tableName,TeAttributeList& attList)
{
	vector<string> pks;
	TeDatabasePortal* portal = getPortal();
	if(!portal)
		return false;

	string keys = " SELECT COLUMN_NAME "; 
	keys += " FROM ALL_CONS_COLUMNS, ALL_CONSTRAINTS "; 
	keys += " WHERE ALL_CONS_COLUMNS.CONSTRAINT_NAME = ALL_CONSTRAINTS.CONSTRAINT_NAME ";
	keys += " AND ALL_CONSTRAINTS.table_name = '"+ TeConvertToUpperCase(tableName) +"' AND ";
	keys += " ALL_CONSTRAINTS.OWNER= '"+ TeConvertToUpperCase(user()) +"' AND ";
	keys += " ALL_CONSTRAINTS.CONSTRAINT_TYPE = 'P' ";

	if(!portal->query(keys))
	{
		delete portal;
		return false;
	}

	while(portal->fetchRow())
		pks.push_back(portal->getData(0));

	portal->freeResult();

	string sql = "SELECT * FROM " + tableName + " WHERE 1=2";
	
	if (!portal->query(sql))
	{
		delete portal;
		return false;
	}
	attList = portal->getAttributeList();

	vector<string>::iterator itn;
	TeAttributeList::iterator it = attList.begin();
	while (it != attList.end())
	{
		itn = find(pks.begin(),pks.end(), TeConvertToUpperCase((*it).rep_.name_));
		if (itn != pks.end())
			(*it).rep_.isPrimaryKey_ = true;
		++it;
	}
	delete portal;
	return true;
}

string  
TeOracle::getSQLTime(TeTime& time)
{
	string dateTime = time.getDateTime();	
	string result;
	result = "TO_DATE('";
	result += dateTime;
	result += "  ";
	result += "','DD/MM/YYYY HH24:MI:SS')"; 
	return result;
}


//----- TeOraclePortal methods ---

TeOraclePortal::TeOraclePortal ( TeDatabase*  pDatabase) 
{
	db_ = pDatabase;
	connection_ = ((TeOracle*)pDatabase)->connection_;
	recset_.CreateInstance(__uuidof(ADODB::Recordset));
}


TeOraclePortal::TeOraclePortal()
{
	recset_.CreateInstance(__uuidof(ADODB::Recordset));
	curRow_ = 0;
}

TeOraclePortal::~TeOraclePortal ()
{
	freeResult();
}


bool 
TeOraclePortal::query (const string &qry, TeCursorLocation l, TeCursorType /* t */, TeCursorEditType /* e */, TeCursorDataType /*dt */ )
{
	try
	{
		if(l==TeCLIENTESIDE)
			recset_->CursorLocation=ADODB::adUseClient;
		else
			recset_->CursorLocation=ADODB::adUseServer;

		recset_->Open(qry.c_str(),connection_.GetInterfacePtr(),
					ADODB::adOpenForwardOnly,  //adOpenKeyset  //adOpenStatic
					ADODB::adLockOptimistic,
					ADODB::adCmdText);
		numRows_ = recset_->GetRecordCount();
	}
	catch(_com_error &e)
	{
		errorMessage_ = e.Description();
		return false;
	}
	catch(...)
	{
		errorMessage_ = "Oppps !";
		return false;
	}

	ADODB::FieldsPtr ptrFields = NULL;
	ADODB::FieldPtr ptrField = NULL;
	ptrFields = recset_->Fields;
	numFields_ = ptrFields->Count; 

	attList_.clear ();
	int i;
	for(i = 0; i < numFields_; i++)
	{
		_variant_t vCol((long)i);
//		HRESULT hr = ptrFields->get_Item(vCol, &ptrField);
		ptrFields->get_Item(vCol, &ptrField);
		TeAttribute attribute;
		
		int nType = ptrField->Type;
//		int precision = ptrField->Precision;
		int numericScale = ptrField->NumericScale;

		if((nType==ADODB::adNumeric)||(nType==ADODB::adVarNumeric))
		{
			if(numericScale>0)
				nType = ADODB::adDouble;
			else
				nType = ADODB::adInteger;
		}
		
		switch (nType)
		{
		case ADODB::adBoolean:
		case ADODB::adNumeric:
		case ADODB::adInteger:
		case ADODB::adSmallInt:
		case ADODB::adTinyInt:
		case ADODB::adUnsignedInt:
		case ADODB::adUnsignedSmallInt:
		case ADODB::adUnsignedTinyInt:
		case ADODB::adVarNumeric: 
			attribute.rep_.type_ = TeINT;
			break;

		case ADODB::adSingle:
		case ADODB::adDouble:
			attribute.rep_.type_ = TeREAL;
			attribute.rep_.decimals_ = numericScale; 
			break;

		case ADODB::adDate:
		case ADODB::adDBDate:
		case ADODB::adDBTime:
		case ADODB::adDBTimeStamp:
			attribute.rep_.type_ = TeDATETIME;
			attribute.dateChronon_ = TeSECOND;
			attribute.dateTimeFormat_ = TeAdo::systemDateTimeFormat(attribute.indicatorAM_, attribute.indicatorPM_, attribute.dateSeparator_, attribute.timeSeparator_);
			break;

		case ADODB::adLongVarBinary:
		case ADODB::adVarBinary:
		case ADODB::adBinary:
			attribute.rep_.type_ = TeBLOB;
			break;

		case ADODB::adChar: 
		case ADODB::adWChar:
		case ADODB::adVarChar:
		case ADODB::adVarWChar:
		case ADODB::adLongVarChar:
		case ADODB::adLongVarWChar: 
			attribute.rep_.type_ = TeSTRING;
			break;

		case ADODB::adCurrency: 
		default :
			attribute.rep_.type_ = TeUNKNOWN;
			break;
		}
		attribute.rep_.name_ = ptrField->Name;
		attribute.rep_.numChar_ = ptrField->DefinedSize;
		attList_.push_back ( attribute );
	}

	curRow_=-1;

	return true;
}

int 
TeOraclePortal ::getInt (int i) 
{
	int value = atoi(getData(i));
	return value;
}

int  
TeOraclePortal::getInt (const string& s)
{
	int value = atoi(getData(s));
	return value;
}



TeTime
TeOraclePortal::getDate (const string& name)
{
	_variant_t value;

	TeTime t;
	try
	{
		value = recset_->GetCollect(name.c_str());
	}
	catch(_com_error &e)
	{
		string field = TeGetExtension(name.c_str());
		if (!field.empty())
		{
			try
			{
				value = recset_->GetCollect(field.c_str());
			}
			catch(_com_error &e)
			{
				errorMessage_ = e.Description();
				return t;
			}
			catch(...)
			{
				errorMessage_ = "Error getDate!";
				return t;
			}
		}
		else
		{
			errorMessage_ = e.Description();
			return t;
		}
	}
	catch(...)
	{
		errorMessage_ = "Oppps !";
		return t;
	}

	if (value.vt != VT_NULL)
	{
		bvalue_ = _bstr_t(value);
		TeAdo* adoDb = (TeAdo*) this->getDatabase();
		t = TeTime (string((char*)bvalue_), TeSECOND, adoDb->systemDateTimeFormat_, 
			adoDb->systemDateSep_, adoDb->systemTimeSep_, adoDb->systemIndPM_);
	}
	return t;
}


TeTime
TeOraclePortal::getDate (int i)
{
    _variant_t vtIndex;
	_variant_t value;

    vtIndex.vt = VT_I4;
	vtIndex.lVal = i;
	
	TeTime t;
	try
	{
		value = recset_->GetFields()->GetItem(vtIndex)->Value;
	}
	catch(_com_error &e)
	{
		errorMessage_ = e.Description();
		return t;
	}
	catch(...)
	{
		errorMessage_ = "Oppps !";
		return t;
	}
	if (value.vt != VT_NULL)
	{
		bvalue_ = _bstr_t(value);
		TeAdo* adoDb = (TeAdo*) this->getDatabase();
		t = TeTime (string((char*)bvalue_), TeSECOND, adoDb->systemDateTimeFormat_, 
			adoDb->systemDateSep_, adoDb->systemTimeSep_, adoDb->systemIndPM_);
	}
	return t;
}


string
TeOraclePortal::getDateAsString(int i)
{
	TeTime t = this->getDate(i);
	string date = t.getDateTime ();

	if (!date.empty())
	{		string tval = " TO_DATE ('"+ date +"','DDsMMsYYYYsHHsmmsSS') ";
		return tval;
	}
	else
		return "";
}

string 
TeOraclePortal::getDateAsString(const string& s)
{
	TeTime t = this->getDate(s);
	string date = t.getDateTime ();

	if (!date.empty())
	{		string tval = " TO_DATE ('"+ date +"','DDsMMsYYYYsHHsmmsSS') ";
		return tval;
	}
	else
		return "";
}



