/************************************************************************************
TerraLib - a library for developing GIS applications.
Copyright  2001-2004 INPE and Tecgraf/PUC-Rio.

This code is part of the TerraLib library.
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

You should have received a copy of the GNU Lesser General Public
License along with this library.

The authors reassure the license terms regarding the warranties.
They specifically disclaim any warranties, including, but not limited to,
the implied warranties of merchantability and fitness for a particular purpose.
The library provided hereunder is on an "as is" basis, and the authors have no
obligation to provide maintenance, support, updates, enhancements, or modifications.
In no event shall INPE and Tecgraf / PUC-Rio be held liable to any party for direct,
indirect, special, incidental, or consequential damages arising out of the use
of this library and its documentation.
*************************************************************************************/
/*! \file TeAdoSqlServer.h
    This file particularities of a ADO SQL Server driver
*/
#ifndef  __TERRALIB_INTERNAL_ADOSQLSERVER_H
#define  __TERRALIB_INTERNAL_ADOSQLSERVER_H

#include <TeDatabaseFactory.h>

#ifdef WIN32
#pragma warning ( disable: 4192 )
#pragma warning ( disable: 4146 )
#include <atldbcli.h>
#include <iostream>
#include <comdef.h>

/*
 *   Lets use the import directive to create some smart pointers for us
 */
#import "msado26.tlb"  rename("EOF", "IsEOF") rename("BOF", "IsBOF")
#import <msadox.dll>
#import <oledb32.dll> rename_namespace("OLEDB") 
#else
#error ONLY FOR WINDOWS OPERATIONAL SYSTEM
#endif

#include <TeDatabase.h>
#include <TeAdoDB.h>


#ifdef AFX_DLL
#define EXPORT_WIN __declspec( dllexport )
#else
#define EXPORT_WIN
#endif
//! A driver class to an Sql Server database accessible using ADO library
/*!
  \sa
  TeDatabase
*/
class  EXPORT_WIN  TeSqlServer : public TeAdo
{
friend class TeSqlServerPortal;

public: 

	TeSqlServer();
	~TeSqlServer();

	// Methods to create and connect to a ADO database
		bool newDatabase(const string& database, const string& user, const string& password, const string& host, const int &port=0, bool terralibModel=true);
		bool connect (const string& host, const string& user, const string& password, const string& database, int port = 0);

	// Methods to manipulate a table
		bool tableExist(const string& table);
     	bool columnExist(const string& table, const string& column, TeAttribute& attr);
		bool getAttributeList(const string& tableName,TeAttributeList& attList);
     	bool createTable(const string& table, TeAttributeList &attr);
		bool alterTable(const string& oldTableName, const string& newTablename);
			
     	bool addColumn(const string& table,TeAttributeRep &rep);
		bool deleteTable (const string& table);
     	bool createRelation (const string& name, const string& table, const string& fieldName, const string& relatedTable, const string& relatedField, bool cascadeDeletion);

		bool deleteLayer(int layerId);
		bool deleteTheme(int themeId);
		bool deleteView (int viewId);

		TeDBRelationType existRelation(const string& tableName, const string& relName);

	// Methods to create the data model tables
		bool createDatabaseTable();
     	bool createProjectionTable ();
     	bool createLayerTable ();
     	bool createRepresentationTable ();
     	bool createRasterMetadataTable(const string& tableName);
     	bool createViewTable ();
     	bool createThemeTable ();
		bool createGroupingTable();
     	bool createThemeTablesTable();
     	bool createLegendTable ();
     	bool createCollectionTable(const string& tableName);

     	bool createLayerTableTable();
     	bool createTablesRelationTable();

	// Methods to create the geometry tables
     	bool createMosaicTable (const string& table);
     	bool createCellGeometry (const string& table);
     	bool createTextGeometry (const string& table);
     	bool createArcGeometry (const string& table);
     	bool createNodeGeometry (const string& table);
     	bool createRasterGeometry (const string& table);
		bool createRasterTable (const string& tableName);
     	bool createPolygonGeometry (const string& table);
     	bool createLineGeometry (const string& table);
     	bool createPointGeometry (const string& table);
     	bool createLUTTable(const string& table);

	// Insert/Update methods

	// Insert an attribute table
     	bool insertTheme (TeTheme *theme);     	
		bool generateLabelPositions (TeTheme *theme);
    
		bool insertPolygonSet (const string& table, TePolygonSet &ps);	
     	bool insertPolygon (const string& table, TePolygon &p);

     	TeDatabasePortal* getPortal ();
     	bool createVisualTable ();
		bool createVisualRasterTable();

		string getSQLTime(TeTime& time); 

};

//! A portal to access a ADO database
class  EXPORT_WIN  TeSqlServerPortal : public TeAdoPortal
{

public :

	// Constructor / Destructor
	TeSqlServerPortal ();
	TeSqlServerPortal (TeDatabase* pDatabase);
	~TeSqlServerPortal ();

	TeTime getDate (int i);
	TeTime getDate (const string& s);

	string getDateAsString(int i);
	string getDateAsString(const string& s);
};


/**
 * @brief This is the class for TeSqlServer driver factory.
 * @author Emiliano F. Castejon <castejon@dpi.inpe.br>
 * @ingroup DatabaseUtils
 */
class TeSqlServerFactory : public TeDatabaseFactory
{
  public :
      
    /**
     * Default constructor
     */
    TeSqlServerFactory() : TeDatabaseFactory( 
      std::string( "SqlServerAdo" ) ) {};      
      
    /**
     * Default Destructor
     */
    ~TeSqlServerFactory() {};
      
  protected :  
  
    /**
     * Implementation for the abstract TeFactory::build.
     *
     * @param arg A const reference to the parameters used by the
     * database.
     * @return A pointer to the new generated database instance.
     */
    TeDatabase* build( const TeDatabaseFactoryParams& arg )
    {
      TeSqlServer* instance_ptr = new TeSqlServer();
      
      if( arg.host_ != "" ) {
        instance_ptr->connect( arg.host_, arg.user_, arg.password_,
          arg.database_, arg.port_ );
      }
  
      return (TeDatabase*)instance_ptr;
    }
};

namespace {
  static TeSqlServerFactory TeSqlServerFactory_instance;
};


#endif
